# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_occurrence.

This module contains unit tests for abydos.fingerprint.Occurrence
"""

import unittest

from abydos.fingerprint import Occurrence


class OccurrenceFingerprintTestCases(unittest.TestCase):
    """Test Cisłak & Grabowski's occurrence fingerprint functions.

    abydos.fingerprint.Occurrence
    """

    fp = Occurrence()

    def test_occurrence_fingerprint(self):
        """Test abydos.fingerprint.Occurrence."""
        # Base case
        self.assertEqual(self.fp.fingerprint(''), '0' * 16)

        # https://arxiv.org/pdf/1711.08475.pdf
        self.assertEqual(self.fp.fingerprint('instance'), '1110111000010000')

        self.assertEqual(self.fp.fingerprint('inst'), '0100111000000000')
        self.assertEqual(
            Occurrence(15).fingerprint('instance'), '111011100001000'
        )
        self.assertEqual(
            Occurrence(32).fingerprint('instance'),
            '11101110000100000000000000000000',
        )
        self.assertEqual(
            Occurrence(64).fingerprint('instance'),
            '11101110000100000000000000000000' + '0' * 32,
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_haase.

This module contains unit tests for abydos.phonetic.Haase
"""

import unittest

from abydos.phonetic import Haase


class HaaseTestCases(unittest.TestCase):
    """Test Haase Phonetik functions.

    test cases for abydos.phonetic.Haase
    """

    pa = Haase()

    def test_haase_phonetik(self):
        """Test abydos.phonetic.Haase."""
        # Base cases
        self.assertEqual(self.pa.encode(''), '')

        # equivalents
        self.assertEqual(self.pa.encode('Häschen'), self.pa.encode('Haeschen'))
        self.assertEqual(self.pa.encode('Schloß'), self.pa.encode('Schloss'))
        self.assertEqual(self.pa.encode('üben'), self.pa.encode('ueben'))
        self.assertEqual(
            self.pa.encode('Eichörnchen'), self.pa.encode('Eichoernchen')
        )

        # coverage completion
        self.assertEqual(self.pa.encode('Häschen'), '9896,9496')
        self.assertEqual(Haase(primary_only=True).encode('Häschen'), '9896')
        self.assertEqual(self.pa.encode('Eichörnchen'), '94976496')
        self.assertEqual(self.pa.encode('Hexe'), '9489')
        self.assertEqual(self.pa.encode('Chemie'), '4969,8969')

        self.assertEqual(self.pa.encode('Brille'), '17959,179')
        self.assertEqual(self.pa.encode('Brilleille'), '1795959,17959,179')
        self.assertEqual(self.pa.encode('Niveau'), '6939')
        self.assertEqual(self.pa.encode('Korb'), '4971,4973')
        self.assertEqual(self.pa.encode('Heino'), '969,9693')
        self.assertEqual(self.pa.encode('Nekka'), '6949,69497')
        self.assertEqual(self.pa.encode('Aleph'), '9593')
        self.assertEqual(self.pa.encode('Aleppo'), '95919,959193')
        self.assertEqual(self.pa.encode('Endzipfel'), '96891395')
        self.assertEqual(self.pa.encode('verbrandt'), '39717962,39737962')
        self.assertEqual(self.pa.encode('Cent'), '8962')
        self.assertEqual(self.pa.encode('addiscendae'), '92989629')
        self.assertEqual(self.pa.encode('kickx'), '4948')
        self.assertEqual(self.pa.encode('sanctionen'), '896829696')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('Niveau'), 'NAFA')
        self.assertEqual(self.pa.encode_alpha('Korb'), 'KARP,KARF')
        self.assertEqual(self.pa.encode_alpha('Heino'), 'ANA,ANAF')
        self.assertEqual(self.pa.encode_alpha('Nekka'), 'NAKA,NAKAR')


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_dolby.

This module contains unit tests for abydos.phonetic.Dolby
"""

import unittest

from abydos.phonetic import Dolby


class DolbyTestCases(unittest.TestCase):
    """Test Dolby functions.

    test cases for abydos.phonetic.Dolby
    """

    pa = Dolby()

    def test_dolby(self):
        """Test abydos.phonetic.Dolby."""
        # Base case
        self.assertEqual(self.pa.encode(''), '')

        # Tests from Dolby (1970) pp. 264--274
        # https://ejournals.bc.edu/ojs/index.php/ital/article/download/5259/4734
        # Checked against Cunningham, et al. (1969) pp. 127--136, as needed.
        # https://files.eric.ed.gov/fulltext/ED029679.pdf
        test_cases = (
            ('*BL', 'Abel', 'Abele', 'Abell', 'Able'),
            ('*BRMS', 'Abrahams', 'Abrams'),
            ('*BRMSN', 'Abrahamson', 'Abramson'),
            ('*D', 'Eddy', 'Eddie'),
            ('*DMNS', 'Edmonds', 'Edmunds'),
            ('*DMNSN', 'Edmondson', 'Edmundson'),
            ('*DMS', 'Adams', 'Addems'),
            ('*GN', 'Eagen', 'Egan', 'Eggen'),
            ('*GR', '!Jaeger', 'Yaeger', 'Yeager'),
            ('*KN', 'Aiken', 'Aikin', 'Aitken'),
            ('*KNS', 'Adkins', 'Akins'),
            ('*KR', 'Acker', 'Aker'),
            ('*KR', 'Eckard', 'Eckardt', 'Eckart', 'Eckert', 'Eckhardt'),
            ('*KS', 'Oakes', 'Oaks', 'Ochs'),
            ('*LBRD', 'Albright', 'Allbright'),
            ('*LD', 'Elliot', 'Elliott'),
            ('*LN', 'Allan', 'Allen', 'Allyn'),
            ('*LSN', 'Ohlsen', 'Olesen', 'Olsen', 'Olson', 'Olsson'),
            ('*LVR', 'Oliveira', 'Olivera', 'Olivero'),
            ('*MS', 'Ames', 'Eames'),
            ('*NGL', 'Engel', 'Engle', 'Ingle'),
            ('*NL', "O'Neal", "O'Neil", "O'Neill"),
            ('*NRS', 'Andrews', 'Andrus'),
            ('*NRSN', 'Andersen', 'Anderson', 'Andreasen'),
            ('*NS', 'Ennis', 'Enos'),
            # Corrected: 'Enrichsen' below was an error
            # (It's correct in Cunningham, et al. 1969.)
            (
                '*RKSN',
                'Erichsen',
                'Erickson',
                'Ericson',
                'Ericsson',
                'Eriksen',
            ),
            ('*RL', 'Earley', 'Early'),
            ('*RN', 'Erwin', 'Irwin'),
            (
                '*RNS',
                'Aarons',
                'Ahrends',
                'Ahrens',
                'Arens',
                'Arentz',
                'Arons',
            ),
            ('*RS', 'Ayers', 'Ayres'),
            ('*RVN', 'Ervin', 'Ervine', 'Irvin', 'Irvine'),
            ('*RVNG', 'Erving', 'Irving'),
            ('*SBRN', 'Osborn', 'Osborne', 'Osbourne', 'Osburn'),
            ('B*D', 'Beatie', 'Beattie', 'Beatty', 'Beaty', 'Beedie'),
            ('B*DS', 'Betts', 'Betz'),
            ('B*KMN', 'Bachman', 'Bachmann', 'Backman'),
            ('B*L', 'Bailey', 'Baillie', 'Bailly', 'Baily', 'Bayley'),
            ('B*L', 'Beal', 'Beale', 'Beall', 'Biehl'),
            ('B*L', 'Belew', 'Ballou', 'Bellew'),
            ('B*L', 'Buhl', 'Buell'),
            ('B*L', 'Belle', 'Bell'),
            # Corrected: No reason for D to disappear
            ('B*LDN', 'Bolton', 'Boulton'),
            ('B*M', 'Baum', 'Bohm', 'Bohme'),
            ('B*MN', 'Bauman', 'Bowman'),
            ('B*N', 'Bain', 'Bane', 'Bayne'),
            ('B*ND', 'Bennet', 'Bennett'),
            (
                'B*R',
                'Baer',
                'Bahr',
                'Baier',
                'Bair',
                'Bare',
                'Bear',
                'Beare',
                'Behr',
                'Beier',
                'Bier',
                '!Bryer',
            ),
            ('B*R', 'Barry', 'Beare', 'Beery', 'Berry'),
            ('B*R', 'Bauer', 'Baur', 'Bower'),
            ('B*R', 'Bird', 'Burd', 'Byrd'),
            ('B*RBR', 'Barbour', 'Barber'),
            ('B*RG', 'Berg', 'Bergh', 'Burge'),
            ('B*RGR', 'Berger', 'Burger'),
            ('B*RK', 'Boerke', 'Birk', 'Bourke', 'Burk', 'Burke'),
            ('B*RN', 'Burn', 'Byrne'),
            ('B*RNR', 'Bernard', 'Bernhard', 'Bernhardt', 'Bernhart'),
            ('B*RNS', 'Berns', 'Birns', 'Burns', 'Byrns', 'Byrnes'),
            ('B*RNSN', 'Bernstein', 'Bornstein'),
            # Corrected: 'RCH' -> 'RH' in rule 2
            ('B*RS', 'Bertsch', '!Birch', '!Burch'),
            ('BL*KBRN', 'Blackburn', '!Blagburn'),
            ('BL*M', 'Blom', 'Bloom', 'Bluhm', 'Blum', 'Blume'),
            ('BR*D', 'Brode', 'Brodie', 'Brody'),
            ('BR*N', 'Braun', 'Brown', 'Browne'),
            ('BR*N', 'Brand', 'Brandt', 'Brant'),
            # Corrected: 'Diezt' -> 'D*SD', so reversed zt -> tz
            # (Correct in Cunningham, et al. 1969.)
            ('D*DS', 'Dietz', 'Ditz'),
            ('D*F', 'Duffie', 'Duffy'),
            ('D*GN', 'Dougan', 'Dugan', 'Duggan'),
            ('D*K', 'Dickey', 'Dicke'),
            ('D*KNSN', 'Dickenson', '!Dickerson', 'Dickinson', '!Dickison'),
            ('D*KSN', 'Dickson', 'Dixon', 'Dixson'),
            ('D*L', 'Dailey', 'Daily', 'Daley', 'Daly'),
            ('D*L', 'Dahl', 'Dahle', 'Dall', 'Doll'),
            ('D*L', 'Deahl', 'Deal', 'Diehl'),
            ('D*MN', 'Diamond', 'Dimond', 'Dymond'),
            ('D*N', 'Dean', 'Deane', 'Deen'),
            ('D*N', 'Denney', 'Denny'),
            ('D*N', 'Donahoo', 'Donahue', 'Donoho', 'Donohoe', 'Donohoo,'),
            ('D*N', 'Donohue', 'Dunnahoo'),
            ('D*N', 'Downey', 'Downie'),
            ('D*N', 'Dunn', 'Dunne'),
            ('D*NL', 'Donley', 'Donnelley', 'Donnelly'),
            ('D*R', 'Daugherty', 'Doherty', 'Dougherty'),
            ('D*R', 'Dyar', 'Dyer'),
            ('D*RM', 'Derham', 'Durham'),
            ('D*VDSN', 'Davidsen', 'Davidson', '!Davison'),
            ('D*VS', 'Davies', 'Davis'),
            ('DR*SL', 'Driscoll', 'Driskell'),
            ('F*', 'Fay', 'Fahay', 'Fahey'),
            ('F*FR', 'Fifer', 'Pfeffer', 'Pfeiffer'),
            ('F*GN', 'Fagan', 'Feigan', 'Fegan'),
            ('F*L', 'Feil', 'Pfeil'),
            # Corrected: T -> D after LD -> D
            ('F*L', 'Feld', 'Feldt', '!Felt'),
            ('F*LKNR', 'Faulkner', 'Falconer'),
            ('F*LPS', 'Philips', 'Phillips'),
            ('F*NGN', 'Finnegan', 'Finnigan'),
            ('F*NL', 'Finlay', 'Finley'),
            ('F*RL', 'Farrell', 'Ferrell'),
            ('F*RR', 'Ferrara', 'Ferreira', 'Ferriera'),
            # Corrected: No reason for S to be eliminated
            ('F*RSR', 'Foerster', 'Forester', 'Forrester', 'Forster'),
            ('F*RS', 'Forrest', 'Forest'),
            ('F*RS', 'Faris', 'Farriss', 'Ferris', 'Ferriss'),
            ('F*RS', 'First', 'Fuerst', 'Furst'),
            ('F*SR', 'Fischer', 'Fisher'),
            ('FL*N', 'Flinn', 'Flynn'),
            ('FL*NGN', 'Flanagan', 'Flanigan', 'Flannigan'),
            ('FR*', 'Frei', 'Frey', 'Fry', 'Frye'),
            ('FR*DMN', 'Freedman', 'Friedman'),
            # Corrected: Fredickson -> Fredrickson
            # (Correct in Cunningham, et al. 1969.)
            (
                'FR*DRKSN',
                'Frederickson',
                'Frederiksen',
                'Fredrickson',
                'Fredriksson',
            ),
            # Corrected: NK would not reduce to K because of rule 4
            ('FR*NK', 'Franck', 'Frank'),
            ('FR*NS', 'France', 'Frantz', 'Franz'),
            # Corrected: vowel deletion happens after double
            # consonant deletion
            ('FR*NSS', 'Frances', 'Francis'),
            ('FR*S', 'Freeze', 'Freese', 'Fries'),
            ('FR*SR', 'Fraser', 'Frasier', 'Frazer', 'Frazier'),
            ('G*D', 'Good', 'Goode'),
            ('G*DS', 'Getz', 'Goetz', 'Goetze'),
            ('G*F', 'Goff', 'Gough'),
            ('G*L', 'Gold', 'Goold', 'Gould'),
            ('G*LMR', 'Gilmer', 'Gilmore', 'Gilmour'),
            ('G*LR', 'Gallagher', 'Gallaher', 'Galleher'),
            ('G*MS', 'Gomes', 'Gomez'),
            ('G*NR', 'Guenther', 'Gunther'),
            ('G*NSLS', 'Gonzales', 'Gonzalez'),
            # Corrected: 'Consalves' for the first case likely a typo
            # (Correct in Cunningham, et al. 1969.)
            ('G*NSLVS', 'Gonsalves', 'Gonzalves'),
            ('G*RD', 'Garratt', 'Garrett'),
            ('G*RD', 'Garrity', 'Geraghty', 'Geraty', 'Gerrity'),
            ('G*RN', 'Gorden', 'Gordohn', 'Gordon'),
            ('G*RNR', 'Gardiner', 'Gardner', 'Gartner'),
            ('G*RR', 'Garrard', 'Gerard', 'Gerrard', 'Girard'),
            ('G*S', 'Gauss', 'Goss'),
            ('GR*', 'Gray', 'Grey'),
            ('GR*FD', 'Griffeth', 'Griffith'),
            ('GR*N', 'Green', 'Greene'),
            ('GR*S', 'Gros', 'Grose', 'Gross'),
            ('H*D', 'Hyde', 'Heidt'),
            ('H*F', 'Hoff', 'Hough', 'Huff'),
            ('H*FMN', 'Hoffman', 'Hoffmann', 'Hofman', 'Hofmann', 'Huffman'),
            ('H*G', 'Hoag', 'Hoge', 'Hogue'),
            ('H*GN', 'Hagan', 'Hagen'),
            ('H*K', 'Hauch', 'Hauck', 'Hauk', 'Hauke'),
            ('H*KSN', 'Hutcheson', 'Hutchison'),
            ('H*L', 'Holley', 'Holly'),
            ('H*L', 'Holl', 'Hall'),
            ('H*L', 'Halley', 'Haley'),
            ('H*L', 'Haile', 'Hale'),
            ('H*LD', 'Holiday', 'Halliday', 'Holladay', 'Holliday'),
            ('H*LG', 'Helwig', 'Hellwig'),
            ('H*LM', 'Holm', '!Home'),
            ('H*LMS', 'Holmes', '!Homes'),
            ('H*LN', 'Highland', 'Hyland'),
            ('H*M', 'Ham', 'Hamm'),
            ('H*MR', 'Hammar', 'Hammer'),
            ('H*N', 'Hanna', 'Hannah'),
            ('H*N', 'Hahn', 'Hahne', 'Hann', 'Haun'),
            ('H*NN', 'Hanan', 'Hannan', 'Hannon'),
            ('H*NRKS', 'Hendricks', 'Hendrix', 'Henriques'),
            ('H*NRKSN', 'Hendrickson', 'Henriksen', 'Henrikson'),
            (
                'H*NS',
                'Heintz',
                'Heinz',
                'Heinze',
                'Hindes',
                'Hinds',
                'Hines',
                'Hinze',
            ),
            ('H*NS', 'Haines', 'Haynes'),
            (
                'H*NSN',
                'Henson',
                'Hansen',
                'Hanson',
                'Hanssen',
                'Hansson',
                'Hanszen',
            ),
            ('H*R', 'Herd', 'Heard', 'Hird', 'Hurd'),
            ('H*R', 'Hart', 'Hardt', 'Harte', 'Heart'),
            ('H*R', 'Hare', 'Hair'),
            ('H*R', 'Hardey', 'Hardie', 'Hardy'),
            ('H*RMN', 'Hartman', 'Hardmen', 'Hardman', 'Hartmann'),
            ('H*RMN', 'Herman', 'Hermann', 'Herrmann'),
            ('H*RMN', 'Harman', 'Harmon'),
            ('H*RN', 'Heron', 'Herrin', 'Herron'),
            ('H*RN', 'Hardin', 'Harden'),
            ('H*RN', 'Horn', 'Horne'),
            ('H*RNGDN', 'Herrington', 'Harrington'),
            ('H*S', 'Haas', 'Haase', 'Hasse'),
            ('H*S', 'Howes', 'House', 'Howse'),
            ('H*S', 'Hays', 'Hayes'),
            ('H*SN', 'Houston', 'Huston'),
            ('H*VR', 'Hoover', 'Hover'),
            ('J*', 'Jew', 'Jue'),
            ('J*FR', 'Jeffery', 'Jeffrey'),
            ('J*FRS', 'Jefferies', 'Jefferis', 'Jefferys', 'Jeffreys'),
            ('J*KB', 'Jacobi', 'Jacoby'),
            ('J*KBSN', 'Jacobsen', 'Jacobson', 'Jackobsen'),
            ('J*KS', 'Jacques', 'Jacks', 'Jaques'),
            ('J*L', 'Jewell', 'Juhl'),
            ('J*MS', 'Jaimes', 'James'),
            ('J*MSN', 'Jameson', 'Jamieson', 'Jamison'),
            (
                'J*NSN',
                'Jahnsen',
                'Jansen',
                'Jansohn',
                'Janssen',
                'Jansson,',
                'Janzen',
                'Jensen',
                'Jenson',
            ),
            ('J*S', 'Joice', 'Joyce'),
            ('K*', 'Kay', 'Kaye'),
            ('K*F', 'Coffee', 'Coffey'),
            ('K*FMN', 'Coffman', 'Kauffman', 'Kaufman', 'Kaufmann'),
            ('K*K', 'Cook', 'Cooke', 'Koch', 'Koche'),
            ('K*K', 'Cook', 'Cooke', 'Koch', 'Koche'),
            ('K*L', 'Cole', 'Kohl', 'Koll'),
            ('K*L', 'Cole', 'Kohl', 'Koll'),
            ('K*L', 'Kelley', 'Kelly'),
            ('K*LMN', 'Coleman', 'Colman'),
            ('K*LR', 'Koehler', 'Koeller', 'Kohler', 'Koller'),
            ('K*MBRLN', 'Chamberlain', 'Chamberlin'),
            ('K*MBS', 'Combs', 'Coombes', 'Coombs'),
            ('K*MP', 'Camp', 'Kampe', 'Kampf'),
            ('K*MPS', 'Campos', 'Campus'),
            ('K*N', 'Cahn', 'Conn', 'Kahn'),
            ('K*N', 'Cahen', 'Cain', 'Caine', 'Cane', 'Kain', 'Kane'),
            ('K*N', 'Chin', 'Chinn'),
            ('K*N', 'Chaney', 'Cheney'),
            ('K*N', 'Coen', 'Cohan', 'Cohen', 'Cohn', 'Cone', 'Koehn', 'Kahn'),
            ('K*N', 'Coon', 'Kuhn', 'Kuhne'),
            ('K*N', 'Kenney', 'Kenny', 'Kinney'),
            ('K*NL', 'Conley', 'Conly', 'Connelly', 'Connolly'),
            ('K*NR', 'Conner', 'Connor'),
            ('K*NS', 'Coons', 'Koontz', 'Kuhns', 'Kuns', 'Kuntz', 'Kunz'),
            ('K*P', 'Coop', 'Co-op', 'Coope', 'Coupe', 'Koop'),
            (
                'K*PL',
                'Chapel',
                'Chapell',
                'Chappel',
                'Chappell',
                'Chappelle',
                'Chapple',
            ),
            ('K*R', 'Carrie', 'Carey', 'Cary'),
            ('K*R', 'Corey', 'Cory'),
            ('K*R', 'Carr', 'Kar', 'Karr'),
            # Corrected: No reason to strip S
            ('K*RS', 'Kurtz', 'Kurz'),
            ('K*R', 'Kehr', 'Ker', 'Kerr'),
            ('K*RD', 'Cartwright', 'Cortright'),
            # Corrected: No reason to strip D
            ('K*RLDN', 'Carleton', 'Carlton'),
            # Corrected: CE -> SE
            ('K*RN', 'Carney', '!Cerney', 'Kearney'),
            # Corrected: RC -> R
            ('K*RSNR', 'Kirschner', '!Kirchner'),
            ('K*S', 'Chace', 'Chase'),
            ('K*S', 'Cass', 'Kass'),
            ('K*S', 'Kees', 'Keyes', 'Keys'),
            ('K*SL', 'Cassel', 'Cassell', 'Castle'),
            ('K*SLR', 'Kesler', 'Kessler', 'Kestler'),
            (
                'K*SR',
                'Kaiser',
                'Kayser',
                'Keizer',
                'Keyser',
                'Kieser',
                'Kiser',
                'Kizer',
            ),
            ('KL*N', 'Cline', 'Klein', 'Kleine', 'Kline'),
            ('KL*RK', 'Clark', 'Clarke'),
            ('KL*SN', 'Claussen', 'Clausen', 'Clawson', 'Closson'),
            ('KR*', 'Crow', 'Crowe'),
            ('KR*GR', 'Krieger', 'Kroeger', 'Krueger', 'Kruger'),
            ('KR*MR', 'Creamer', 'Cramer', 'Kraemer', 'Kramer', 'Kremer'),
            ('KR*N', 'Craine', 'Crane'),
            ('KR*S', 'Christie', 'Christy', 'Kristee'),
            ('KR*S', 'Crouss', 'Kraus', 'Krausch', 'Krause', 'Krouse'),
            ('KR*S', 'Cross', 'Krost'),
            ('KR*S', 'Crews', 'Cruz', 'Kruse'),
            ('KR*SNSN', 'Christensen', 'Christiansen', 'Christianson'),
            ('L*', 'Loe', 'Loewe', 'Low', 'Lowe'),
            ('L*', 'Lea', 'Lee', '!Leigh'),
            ('L*D', 'Lloyd', 'Loyd'),
            ('L*DL', 'Litle', 'Littell', 'Little', 'Lytle'),
            ('L*DRMN', 'Ledterman', 'Letterman'),
            ('L*K', 'Leach', 'Leech', 'Leitch'),
            ('L*KS', 'Lucas', 'Lukas'),
            ('L*LN', 'Laughlin', 'Loughlin'),
            ('L*LR', 'Lawler', 'Lawlor'),
            ('L*MB', 'Lamb', '!Lamm'),
            ('L*MN', 'Lemen', 'Lemmon', 'Lemon'),
            ('L*MN', 'Layman', 'Lehman', 'Lehmann'),
            ('L*N', 'Lind', 'Lynd', 'Lynde'),
            ('L*N', 'Lion', 'Lyon'),
            ('L*N', 'Lin', 'Linn', 'Lynn', 'Lynne'),
            # Corrected: NG -> NG (!N)
            ('L*N', 'Lain', 'Laine', '!Laing', 'Lane', 'Layne'),
            ('L*NG', 'Lang', 'Lange'),
            ('L*NN', 'London', 'Lundin'),
            ('L*NS', 'Lindsay', 'Lindsey', '!Lindsley', '!Linsley'),
            ('L*R', 'Lawry', 'Lowery', 'Lowrey', 'Lowry'),
            ('L*RNS', 'Lawrence', 'Lowrance'),
            ('L*RNS', 'Laurence', 'Lawrance', 'Lawrence', 'Lorence', 'Lorenz'),
            ('L*RSN', 'Larsen', 'Larson'),
            ('L*S', 'Lewis', 'Louis', 'Luis', 'Luiz'),
            ('L*S', 'Lacey', 'Lacy'),
            ('L*SR', '!Leicester', 'Lester'),
            ('L*V', 'Levey', 'Levi', 'Levy'),
            ('L*VD', 'Leavett', 'Leavitt', 'Levit'),
            ('L*VL', 'Lavell', 'Lavelle', 'Leavelle', 'Loveall', 'Lovell'),
            ('L*VN', 'Lavin', 'Levin', 'Levine'),
            ('M*D', 'Mead', 'Meade'),
            # Corrected: RT*N -> R*N -> RN
            ('M*RN', '!Moretton', 'Morton'),
            ('M*DS', 'Mathews', 'Matthews'),
            (
                'M*DSN',
                'Madison',
                'Madsen',
                'Matson',
                'Matteson',
                'Mattison',
                'Mattson',
            ),
            ('M*KL', 'Michael', 'Michel'),
            ('M*KM', 'Meacham', 'Mechem'),
            # Corrected: RQ*S -> RKS, not KS
            ('M*RKS', 'Marques', 'Marquez', 'Marquis', 'Marquiss'),
            # Corrected: RKS does not compress to KS
            ('M*RKS', 'Marcks', 'Marks', 'Marx'),
            ('M*LN', 'Maloney', 'Moloney', 'Molony'),
            ('M*LN', 'Mullan', 'Mullen', 'Mullin'),
            ('M*LR', 'Mallery', 'Mallory'),
            ('M*LR', 'Moeller', 'Moller', 'Mueller', 'Muller'),
            ('M*LR', 'Millar', 'Miller'),
            ('M*LS', 'Miles', 'Myles'),
            ('M*N', 'Mahan', 'Mann'),
            ('M*NR', 'Miner', 'Minor'),
            ('M*NR', 'Monroe', 'Munro'),
            ('M*NSN', 'Monson', 'Munson'),
            ('M*R', 'Murray', 'Murrey'),
            ('M*R', 'Maher', 'Maier', 'Mayer'),
            ('M*R', 'Mohr', 'Moor', 'Moore'),
            # Corrected: No reason to eliminate final S
            ('M*RS', 'Meyers', 'Myers'),
            ('M*R', 'Meier', 'Meyer', 'Mieir', 'Myhre'),
            ('M*RF', 'Murphey', 'Murphy'),
            ('M*RL', 'Merrell', 'Merrill'),
            ('M*RN', 'Marten', 'Martin', 'Martine', 'Martyn'),
            ('M*RS', 'Meyers', 'Myers'),
            ('M*RS', 'Maurice', 'Morris', 'Morse'),
            ('MK*', 'McCoy', 'McCaughey'),
            ('MK*', 'Magee', 'McGee', 'McGehee', 'McGhie'),
            ('MK*', 'Mackey', 'MacKay', 'Mackie', 'McKay'),
            ('MK*', 'McCue', '!McHugh'),
            ('MK*L', 'Magill', 'McGill'),
            ('MK*LF', 'McCollough', '!McCullah', 'McCullough'),
            ('MK*LM', 'McCallum', 'McCollum', 'McColm'),
            ('MK*N', 'McKenney', 'McKinney'),
            ('MK*NR', 'Macintyre', 'McEntire', 'Mcintire', 'Mcintyre'),
            ('MK*NS', 'MacKenzie', 'McKenzie'),
            (
                'MK*NS',
                'Maginnis',
                'McGinnis',
                'McGuinness',
                'Mcinnes',
                'Mcinnis',
            ),
            ('MK*R', 'Maguire', 'McGuire'),
            ('MK*R', 'McCarthy', 'McCarty'),
            ('MKD*NL', 'MacDonald', 'McDonald', 'McDonnell'),
            ('MKF*RLN', 'MacFarland', 'MacFarlane', 'McFarland', 'McFarlane'),
            ('MKF*RSN', 'MacPherson', 'McPherson'),
            ('MKL*D', 'MacLeod', 'McCloud', 'McLeod'),
            (
                'MKL*KLN',
                'MacLachlan',
                'Maclachlin',
                'McLachlan',
                '!McLaughlin,',
                '!McLoughlin',
            ),
            ('MKL*LN', 'McClellan', 'McClelland', 'McLellan'),
            ('MKL*N', 'McClain', 'McClaine', 'McLain', 'McLane'),
            ('MKL*N', 'MacLean', 'McClean', 'McLean'),
            ('MKL*S', 'McCloskey', 'McClosky', 'McCluskey'),
            ('MKM*LN', 'MacMillan', 'McMillan', 'McMillin'),
            ('MKN*L', 'MacNeal', 'McNeal', 'McNeil', 'McNeill'),
            ('MKR*D', 'Magrath', 'McGrath'),
            (
                'N*KL',
                'Nichol',
                'Nicholl',
                'Nickel',
                'Nickle',
                'Nicol',
                'Nicoll',
            ),
            ('N*KLS', 'Nicholls', 'Nichols', 'Nickels', 'Nickles', 'Nicols'),
            ('N*KLS', 'Nicholas', 'Nicolas'),
            ('N*KLSN', 'Nicholsen', 'Nicholson', 'Nicolaisen', 'Nicolson'),
            ('N*KSN', 'Nickson', 'Nixon'),
            ('N*L', 'Neal', 'Neale', 'Neall', 'Neel', 'Neil', 'Neill'),
            (
                'N*LSN',
                'Neilsen',
                'Neilson',
                'Nelsen',
                'Nelson',
                'Nielsen',
                'Nielson,',
                'Nilson',
                'Nilssen',
                'Nilsson',
            ),
            ('N*MN', 'Neumann', 'Newman'),
            ('N*RS', 'Norris', 'Nourse'),
            ('N*SBD', 'Nesbit', 'Nesbitt', 'Nisbet'),
            ('P*D', 'Pettee', 'Petty'),
            (
                'P*DRSN',
                'Peterson',
                'Pederson',
                'Pedersen',
                'Petersen',
                'Petterson',
            ),
            ('P*G', 'Page', 'Paige'),
            ('P*LK', 'Polak', 'Pollack', 'Pollak', 'Pollock'),
            ('P*LSN', 'Polson', 'Paulsen', 'Paulson', 'Poulsen', 'Poulsson'),
            ('P*N', 'Paine', 'Payn', 'Payne'),
            ('P*R', 'Parry', 'Perry'),
            ('P*R', 'Parr', 'Paar'),
            ('P*RK', 'Park', 'Parke'),
            ('P*RKS', 'Parks', 'Parkes'),
            # Corrected: RC -> R
            ('P*R', 'Pierce', 'Pearce', 'Peirce', '!Piers'),
            ('P*RS', 'Parish', 'Parrish'),
            ('P*RS', 'Paris', 'Parris'),
            ('P*RSN', 'Pierson', 'Pearson', 'Pehrson', 'Peirson'),
            ('PR*KR', 'Prichard', 'Pritchard'),
            ('PR*NS', 'Prince', 'Prinz'),
            ('PR*R', 'Prior', 'Pryor'),
            ('R*', 'Roe', 'Rowe'),
            ('R*', 'Rae', 'Ray', 'Raye', 'Rea', 'Rey', 'Wray'),
            ('R*BNSN', 'Robinson', '!Robison'),
            ('R*D', 'Rothe', 'Roth'),
            ('R*D', 'Rudd', 'Rood', 'Rude'),
            ('R*D', 'Reed', 'Read', 'Reade', 'Reid'),
            ('R*DR', 'Rider', 'Ryder'),
            ('R*DS', 'Rhoades', 'Rhoads', 'Rhodes'),
            ('R*GN', 'Regan', 'Ragon', 'Reagan'),
            # Corrected: No reason to drop final S
            ('R*GRS', 'Rodgers', 'Rogers'),
            ('R*K', 'Richey', 'Ritchey', 'Ritchie'),
            ('R*K', 'Reich', 'Reiche'),
            ('R*KR', 'Reichardt', 'Richert', 'Rickard'),
            ('R*L', 'Reilley', 'Reilly', 'Reilli', 'Riley'),
            # Corrected: T -> D
            ('R*MNGDN', 'Remington', 'Rimington'),
            ('R*MR', 'Reamer', 'Reimer', 'Riemer', 'Rimmer'),
            ('R*MS', 'Ramsay', 'Ramsey'),
            ('R*N', 'Rhein', 'Rhine', 'Ryan'),
            (
                'R*NR',
                'Reinhard',
                'Reinhardt',
                'Reinhart',
                'Rhinehart',
                'Rinehart',
            ),
            ('R*S', 'Reas', 'Reece', 'Rees', 'Reese', 'Reis', 'Reiss', 'Ries'),
            ('R*S', '!Rauch', 'Rausch', '!Roach', '!Roche', 'Roush'),
            ('R*S', 'Rush', 'Rusch'),
            ('R*S', 'Russ', 'Rus'),
            ('R*VS', 'Reaves', 'Reeves'),
            ('S*BR', 'Seibert', 'Siebert'),
            ('S*FL', 'Schofield', 'Scofield'),
            ('S*FN', 'Stefan', 'Steffan', 'Steffen', 'Stephan', 'Stephen'),
            ('S*FNS', 'Steffens', 'Stephens', '!Stevens'),
            ('S*FNSN', 'Steffensen', 'Steffenson', 'Stephenson', '!Stevenson'),
            (
                'S*FR',
                'Schaefer',
                'Schaeffer',
                'Schafer',
                'Schaffer',
                'Schafer,',
                'Shaffer',
                'Sheaffer',
            ),
            ('S*FR', 'Stauffer', 'Stouffer'),
            ('S*GL', 'Siegal', 'Sigal'),
            ('S*GLR', 'Sigler', 'Ziegler'),
            ('S*K', 'Schuck', 'Shuck'),
            ('S*KS', 'Sachs', 'Sacks', 'Saks', 'Sax', 'Saxe'),
            ('S*L', 'Seeley', 'Seely', 'Seley'),
            ('S*L', 'Schell', 'Shell'),
            ('S*LR', 'Schuler', 'Schuller'),
            # Corrected: LD -> L precedes T -> D
            (
                'S*LDS',
                'Schultz',
                'Schultze',
                '!Schulz',
                '!Schulze',
                'Shults',
                'Shultz',
            ),
            ('S*LV', 'Silva', 'Sylva'),
            ('S*LVR', 'Silveira', 'Silvera', 'Silveria'),
            (
                'S*MKR',
                'Schomaker',
                'Schumacher',
                'Schumaker',
                'Shoemaker,',
                'Shumaker',
            ),
            ('S*MN', 'Simon', 'Symon'),
            ('S*MN', 'Seaman', 'Seemann', 'Semon'),
            ('S*MRS', 'Somers', 'Sommars', 'Sommers', 'Summers'),
            ('S*MS', 'Simms', 'Sims'),
            ('S*N', 'Stein', 'Stine'),
            ('S*N', 'Sweeney', 'Sweeny', 'Sweney'),
            ('S*NR', 'Senter', 'Center'),
            ('S*NRS', 'Sanders', 'Saunders'),
            (
                'S*PR',
                'Shepard',
                '!Shephard',
                '!Shepheard',
                '!Shepherd',
                'Sheppard',
            ),
            ('S*R', 'Stahr', 'Star', 'Starr'),
            ('S*R', 'Stewart', 'Stuart'),
            ('S*R', 'Storey', 'Story'),
            ('S*R', 'Saier', 'Sayre'),
            # Corrected: No reason to strip final S
            ('S*RS', 'Schwartz', 'Schwarz', 'Schwarze', 'Swartz'),
            ('S*RL', 'Schirle', 'Shirley'),
            ('S*RLNG', 'Sterling', 'Stirling'),
            ('S*RMN', 'Scheuermann', 'Schurman', 'Sherman'),
            ('S*RN', 'Stearn', 'Stern'),
            ('S*RR', 'Scherer', 'Shearer', 'Sharer', 'Sherer', 'Sheerer'),
            ('S*S', 'Sousa', 'Souza'),
            ('SM*D', 'Smith', 'Smyth', 'Smythe'),
            ('SM*D', 'Schmid', 'Schmidt', 'Schmit', 'Schmitt', 'Smit'),
            ('SN*DR', 'Schneider', 'Schnieder', 'Snaider', 'Snider', 'Snyder'),
            ('SN*L', 'Schnell', 'Snell'),
            ('SP*LNG', 'Spalding', 'Spaulding'),
            ('SP*R', 'Spear', 'Speer', '!Speirer'),
            # Corrected: No reason to strip final S
            ('SP*RS', 'Spears', 'Speers'),
            ('SR*DR', 'Schroder', 'Schroeder', 'Schroeter'),
            ('SR*DR', 'Schrader', 'Shrader'),
            # Corrected: Everywhere else, rule 3 applies to char 1
            ('D*D', 'Tait', 'Tate'),
            ('D*MSN', 'Thomason', '!Thompson', 'Thomsen', 'Thomson', 'Tomson'),
            ('D*RL', 'Terrel', 'Terrell', 'Terrill'),
            ('DR*S', 'Tracey', 'Tracy'),
            ('V*L', 'Vail', 'Vaile', 'Vale'),
            ('V*L', 'Valley', 'Valle'),
            ('V*R', 'Vieira', 'Vierra'),
            ('W*D', 'White', 'Wight'),
            ('W*DKR', 'Whitacre', 'Whitaker', 'Whiteaker', 'Whittaker'),
            ('W*DL', 'Whiteley', 'Whitley'),
            ('W*DMN', 'Whitman', 'Wittman'),
            ('W*DR', 'Woodard', 'Woodward'),
            ('W*DRS', 'Waters', 'Watters'),
            (
                'W*GNR',
                'Wagener',
                'Waggener',
                'Wagoner',
                'Wagner',
                'Wegner,',
                'Waggoner',
            ),
            ('W*L', 'Willey', 'Willi'),
            ('W*L', 'Wiley', 'Wylie'),
            ('W*L', 'Wahl', 'Wall'),
            ('W*LBR', 'Wilber', 'Wilbur'),
            (
                'W*LF',
                'Wolf',
                'Wolfe',
                'Wolff',
                'Woolf',
                'Woulfe',
                'Wulf',
                'Wulff',
            ),
            ('W*LKNS', 'Wilkens', 'Wilkins'),
            ('W*LKS', 'Wilkes', 'Wilks'),
            ('W*LN', 'Whalen', 'Whelan'),
            # Corrected: LD -> L precedes T -> D
            ('W*LDR', 'Walter', 'Walther', 'Wolter'),
            ('W*LDRS', 'Walters', 'Walthers', 'Wolters'),
            ('W*LS', 'Wallace', 'Wallis'),
            ('W*LS', 'Welch', 'Welsh'),
            ('W*LS', 'Welles', 'Wells'),
            ('W*LSN', 'Willson', 'Wilson'),
            ('W*N', 'Winn', 'Wynn', 'Wynne'),
            ('W*R', 'Worth', 'Wirth'),
            ('W*R', 'Ware', 'Wear', 'Weir', 'Wier'),
            ('W*RL', 'Wehrle', 'Wehrlie', 'Werle', 'Worley'),
            ('W*RNR', 'Warner', 'Werner'),
            ('W*S', 'Weis', 'Weiss', 'Wiese', 'Wise', 'Wyss'),
            (
                'W*SMN',
                'Weismann',
                'Weissman',
                'Weseman',
                'Wiseman,',
                'Wismonn',
                'Wissman',
            ),
        )

        for tests in test_cases:
            result, names = tests[0], tests[1:]
            for name in names:
                if name[0] == '!':
                    self.assertNotEqual(result, self.pa.encode(name[1:]))
                else:
                    self.assertEqual(result, self.pa.encode(name))

        # Additional tests to improve coverage
        self.assertEqual(self.pa.encode('Rune'), 'R*N')
        self.assertEqual(Dolby(keep_vowels=True).encode('Rune'), 'R*N*')
        self.assertEqual(Dolby(vowel_char='').encode('Rune'), 'RN')
        self.assertEqual(Dolby(vowel_char='A').encode('Rune'), 'RAN')
        self.assertEqual(Dolby(max_length=2).encode('Rune'), 'R*')
        self.assertEqual(Dolby(max_length=2).encode('Rune'), 'R*')
        self.assertEqual(Dolby(max_length=4).encode('Wassermann'), 'W*SR')
        self.assertEqual(
            Dolby(max_length=4, keep_vowels=True).encode('Wassermanns'), 'W*S*'
        )
        self.assertEqual(self.pa.encode('Wassermanns'), 'W*SRMNS')

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('Rune'), 'RAN')
        self.assertEqual(self.pa.encode_alpha('Weissman'), 'WASMN')
        self.assertEqual(self.pa.encode_alpha('Pederson'), 'PADRSN')
        self.assertEqual(self.pa.encode_alpha('Frederiksen'), 'FRADRKSN')
        self.assertEqual(self.pa.encode_alpha('Bare'), 'BAR')


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_lig3.

This module contains unit tests for abydos.distance.LIG3
"""

import unittest

from abydos.distance import LIG3


class LIG3TestCases(unittest.TestCase):
    """Test LIG3 functions.

    abydos.distance.LIG3
    """

    cmp = LIG3()

    def test_lig3_sim(self):
        """Test abydos.distance.LIG3.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 1.0)
        self.assertEqual(self.cmp.sim('a', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'a'), 0.0)
        self.assertEqual(self.cmp.sim('a', 'a'), 1.0)
        self.assertEqual(self.cmp.sim('abc', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'abc'), 0.0)
        self.assertEqual(self.cmp.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim('abcd', 'efgh'), 0.0)

        # Testcases from paper
        self.assertEqual(self.cmp.sim('Glavin', 'Glawyn'), 0.8)
        self.assertEqual(
            self.cmp.sim('Williams', 'Vylliems'), 0.7692307692307693
        )
        self.assertEqual(self.cmp.sim('Lewis', 'Louis'), 0.75)
        self.assertEqual(self.cmp.sim('Alex', 'Alexander'), 0.6153846153846154)
        self.assertEqual(self.cmp.sim('Wild', 'Wildsmith'), 0.6153846153846154)
        self.assertEqual(
            self.cmp.sim('Bram', 'Bramberley'), 0.5714285714285714
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_daitch_mokotoff.

This module contains unit tests for abydos.phonetic.DaitchMokotoff
"""

import unittest

from abydos.phonetic import DaitchMokotoff


class DaitchMokotoffTestCases(unittest.TestCase):
    """Test Daitch-Mokotoff Soundex functions.

    test cases for abydos.phonetic.DaitchMokotoff
    """

    pa = DaitchMokotoff()

    def test_daitch_mokotoff(self):
        """Test abydos.phonetic.DaitchMokotoff."""
        # D-M tests
        self.assertEqual(self.pa.encode(''), '000000')

        # http://www.avotaynu.com/soundex.htm
        self.assertEqual(self.pa.encode('Augsburg'), '054795')
        self.assertEqual(self.pa.encode('Breuer'), '791900')
        self.assertEqual(self.pa.encode('Halberstadt'), '587433,587943')
        self.assertEqual(self.pa.encode('Mannheim'), '665600')
        self.assertEqual(self.pa.encode('Chernowitz'), '496740,596740')
        self.assertEqual(self.pa.encode('Cherkassy'), '495400,595400')
        self.assertEqual(self.pa.encode('Kleinman'), '586660')
        self.assertEqual(self.pa.encode('Berlin'), '798600')

        self.assertEqual(self.pa.encode('Ceniow'), '467000,567000')
        self.assertEqual(self.pa.encode('Tsenyuv'), '467000')
        self.assertEqual(self.pa.encode('Holubica'), '587400,587500')
        self.assertEqual(self.pa.encode('Golubitsa'), '587400')
        self.assertEqual(self.pa.encode('Przemysl'), '746480,794648')
        self.assertEqual(self.pa.encode('Pshemeshil'), '746480')
        self.assertEqual(
            self.pa.encode('Rosochowaciec'),
            '944744,944745,944754,944755,945744,945745,945754,945755',
        )
        self.assertEqual(self.pa.encode('Rosokhovatsets'), '945744')

        # https://en.wikipedia.org/wiki/Daitch%E2%80%93Mokotoff_Soundex
        self.assertEqual(self.pa.encode('Peters'), '734000,739400')
        self.assertEqual(self.pa.encode('Peterson'), '734600,739460')
        self.assertEqual(self.pa.encode('Moskowitz'), '645740')
        self.assertEqual(self.pa.encode('Moskovitz'), '645740')
        self.assertEqual(self.pa.encode('Auerbach'), '097400,097500')
        self.assertEqual(self.pa.encode('Uhrbach'), '097400,097500')
        self.assertEqual(
            self.pa.encode('Jackson'), '145460,154600,445460,454600'
        )
        self.assertEqual(
            self.pa.encode('Jackson-Jackson'),
            '145464,145465,154644,154645,154654,445464,445465,454644,'
            + '454645,454654',
        )

        # http://www.jewishgen.org/infofiles/soundex.html
        self.assertEqual(self.pa.encode('OHRBACH'), '097400,097500')
        self.assertEqual(self.pa.encode('LIPSHITZ'), '874400')
        self.assertEqual(self.pa.encode('LIPPSZYC'), '874400,874500')
        self.assertEqual(self.pa.encode('LEWINSKY'), '876450')
        self.assertEqual(self.pa.encode('LEVINSKI'), '876450')
        self.assertEqual(self.pa.encode('SZLAMAWICZ'), '486740')
        self.assertEqual(self.pa.encode('SHLAMOVITZ'), '486740')

        # http://community.actian.com/wiki/OME_soundex_dm()
        self.assertEqual(self.pa.encode('Schwarzenegger'), '474659,479465')
        self.assertEqual(self.pa.encode('Shwarzenegger'), '474659,479465')
        self.assertEqual(self.pa.encode('Schwartsenegger'), '479465')

        # max_length bounds tests
        self.assertEqual(
            DaitchMokotoff(max_length=-1).encode('Niall'), '68' + '0' * 62
        )
        self.assertEqual(
            DaitchMokotoff(max_length=0).encode('Niall'), '680000'
        )

        # zero_pad tests
        self.assertEqual(
            DaitchMokotoff(max_length=-1, zero_pad=False).encode('Niall'),
            '68',
        )
        self.assertEqual(
            DaitchMokotoff(max_length=0, zero_pad=False).encode('Niall'), '68',
        )
        self.assertEqual(
            DaitchMokotoff(max_length=0, zero_pad=True).encode('Niall'),
            '680000',
        )
        self.assertEqual(
            DaitchMokotoff(max_length=6, zero_pad=False).encode(''), '0'
        )
        self.assertEqual(
            DaitchMokotoff(max_length=6, zero_pad=True).encode(''), '000000'
        )

        # encode_alpha
        self.assertEqual(self.pa.encode_alpha('Augsburg'), 'AKSPRK')
        self.assertEqual(self.pa.encode_alpha('Breuer'), 'PRAR')
        self.assertEqual(self.pa.encode_alpha('Halberstadt'), 'KLPSTT,KLPRST')
        self.assertEqual(self.pa.encode_alpha('Mannheim'), 'NNKN')
        self.assertEqual(self.pa.encode_alpha('Chernowitz'), 'SRNPS,KRNPS')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.compression.test_compression_bwt.

This module contains unit tests for abydos.compression.BWT
"""

import unittest

from abydos.compression import BWT


class BWTTestCases(unittest.TestCase):
    """Test abydos.compression.BWT.encode and .decode."""

    coder = BWT()
    coder_pipe = BWT('|')
    coder_dollar = BWT('$')

    def test_bwt_encode(self):
        """Test abydos.compression.BWT.encode."""
        # Examples from Wikipedia entry on BWT
        self.assertEqual(self.coder.encode(''), '\x00')
        self.assertEqual(self.coder_pipe.encode('^BANANA'), 'BNN^AA|A')
        self.assertEqual(
            self.coder_pipe.encode(
                'SIX.MIXED.PIXIES.SIFT.SIXTY.PIXIE.DUST.BOXES'
            ),
            'TEXYDST.E.IXIXIXXSSMPPS.B..E.|.UESFXDIIOIIITS',
        )

        self.assertEqual(self.coder_dollar.encode('aardvark'), 'k$avrraad')

        self.assertRaises(ValueError, self.coder_dollar.encode, 'ABC$')
        self.assertRaises(ValueError, self.coder.encode, 'ABC\0')

    def test_bwt_decode(self):
        """Test abydos.compression.BWT.decode."""
        self.assertEqual(self.coder.decode(''), '')
        self.assertEqual(self.coder.decode('\x00'), '')
        self.assertEqual(self.coder_pipe.decode('BNN^AA|A'), '^BANANA')
        self.assertEqual(
            self.coder_pipe.decode(
                'TEXYDST.E.IXIXIXXSSMPPS.B..E.|.UESFXDIIOIIITS'
            ),
            'SIX.MIXED.PIXIES.SIFT.SIXTY.PIXIE.DUST.BOXES',
        )

        self.assertEqual(self.coder_dollar.decode('k$avrraad'), 'aardvark')

        self.assertRaises(ValueError, self.coder_dollar.decode, 'ABC')
        self.assertRaises(ValueError, self.coder.decode, 'ABC')

    def test_bwt_roundtripping(self):
        """Test abydos.compression.BWT.encode & .decode roundtripping."""
        for w in (
            '',
            'Banana',
            'The quick brown fox, etc.',
            'it is better a chylde unborne than untaught',
            'manners maketh man',
            'בְּרֵאשִׁית, בָּרָא אֱלֹהִים',
            'Ein Rückblick bietet sich folglich an.',
        ):
            self.assertEqual(self.coder.decode(self.coder.encode(w)), w)
            self.assertEqual(
                self.coder_dollar.decode(self.coder_dollar.encode(w)), w
            )


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_aline.

This module contains unit tests for abydos.distance.ALINE
"""

import unittest

from abydos.distance import ALINE


class ALINETestCases(unittest.TestCase):
    """Test ALINE functions.

    abydos.distance.ALINE
    """

    cmp = ALINE()
    cmp_downey = ALINE(normalizer=lambda x: sum(x) / len(x))

    def test_aline_alignments(self):
        """Test abydos.distance.ALINE.alignments."""
        # test cases from Kondrak (2000)
        self.assertEqual(
            self.cmp.alignments('driy', 'tres'),
            [(75.0, '‖ d r iy ‖', '‖ t r e  ‖ s')],
        )
        self.assertEqual(
            self.cmp.alignments('blow', 'flare'),
            [(53.0, '‖ b l o ‖ w', '‖ f l a ‖ re')],
        )
        self.assertEqual(
            self.cmp.alignments('ful', 'plenus'),
            [(48.0, '‖ f u l ‖', '‖ p - l ‖ enus')],
        )
        self.assertEqual(
            self.cmp.alignments('fiz', 'piskis'),
            [(63.0, '‖ f i z ‖', '‖ p i s ‖ kis')],
        )
        self.assertEqual(
            self.cmp.alignments('ay', 'ego'), [(17.5, '‖ ay ‖', '‖ e  ‖ go')]
        )
        self.assertEqual(
            self.cmp.alignments('tuwz', 'dentis'),
            [(75.0, '‖ t uw z ‖', 'den ‖ t i  s ‖')],
        )

        # test cases from Kondrak (2002) after Covington (1996)
        # Some of these alignments are a little different from what's in the
        # thesis because of the differing encoding used.
        self.assertEqual(
            self.cmp.alignments('jo', 'zPe'), [(29.0, '‖ j  o ‖', '‖ zP e ‖')]
        )
        self.assertEqual(
            self.cmp.alignments('tu', 'tuF'), [(45.0, '‖ t u  ‖', '‖ t uF ‖')]
        )
        self.assertEqual(
            self.cmp.alignments('nostros', 'nu'),
            [(47.5, '‖ n o ‖ stros', '‖ n u ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('kyen', 'ki'),
            [(47.5, '‖ k ye ‖ n', '‖ k i  ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('ke', 'kwa'), [(42.5, '‖ k e  ‖', '‖ k wa ‖')]
        )
        self.assertEqual(
            self.cmp.alignments('todos', 'tu'),
            [(47.5, '‖ t o ‖ dos', '‖ t u ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('una', 'uFn'),
            [(45.0, '‖ u  n ‖ a', '‖ uF n ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('dos', 'doF'),
            [(45.0, '‖ d o  ‖ s', '‖ d oF ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('tres', 'trwa'),
            [(77.5, '‖ t r e  ‖ s', '‖ t r wa ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('ombre', 'om'),
            [
                (50.0, '‖ o m ‖ bre', '‖ o m ‖'),
                (50.0, '‖ o mb ‖ re', '‖ o m  ‖'),
            ],
        )
        self.assertEqual(
            self.cmp.alignments('arbol', 'arbreC'),
            [(88.0, '‖ a r b o l ‖', '‖ a r b - r ‖ eC')],
        )
        self.assertEqual(
            self.cmp.alignments('pluFma', 'plum'),
            [(115.0, '‖ p l uF m ‖ a', '‖ p l u  m ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('kabetSa', 'kap'),
            [(75.0, '‖ k a b ‖ etSa', '‖ k a p ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('boka', 'busP'),
            [(68.5, '‖ b o k  ‖ a', '‖ b u sP ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('pye', 'pye'),
            [(65.0, '‖ p y e ‖', '‖ p y e ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('koratSon', 'koFr'),
            [(80.0, '‖ k o  r ‖ atSon', '‖ k oF r ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('ber', 'vwar'),
            [(60.5, '‖ b e  r ‖', '‖ v wa r ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('benir', 'veCnir'),
            [(115.5, '‖ b e  n i r ‖', '‖ v eC n i r ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('detSir', 'dir'),
            [
                (65.0, 'de ‖ tS i r ‖', '‖ d  i r ‖'),
                (65.0, '‖ d e tS i r ‖', '‖ d - -  i r ‖'),
            ],
        )
        self.assertEqual(
            self.cmp.alignments('pobre', 'povreC'),
            [(115.5, '‖ p o b r e  ‖', '‖ p o v r eC ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('dSis', 'diHzes'),
            [(77.5, '‖ dS i s ‖', 'diH ‖ z  e s ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('dSaFt', 'das'),
            [(62.5, '‖ dS aF t ‖', '‖ d  a  s ‖')],
        )
        # Different from paper:
        self.assertEqual(
            self.cmp.alignments('wat', 'vas'),
            [(40.0, 'w ‖ a t ‖', 'v ‖ a s ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('nat', 'nixt'),
            [
                (62.5, '‖ n a - t ‖', '‖ n i x t ‖'),
                (62.5, '‖ n a t  ‖', '‖ n i xt ‖'),
            ],
        )
        self.assertEqual(
            self.cmp.alignments('logN', 'lagN'),
            [(75.0, '‖ l o gN ‖', '‖ l a gN ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('maFn', 'man'),
            [(82.5, '‖ m aF n ‖', '‖ m a  n ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('flesP', 'flaysP'),
            [(122.5, '‖ f l e  sP ‖', '‖ f l ay sP ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('bleCd', 'bluHt'),
            [(99.0, '‖ b l eC d ‖', '‖ b l uH t ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('fedSeCr', 'feHdeCr'),
            [(124.0, '‖ f e  dS eC r ‖', '‖ f eH d  eC r ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('haFr', 'haHr'),
            [(81.5, '‖ h aF r ‖', '‖ h aH r ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('ir', 'oHr'), [(41.5, '‖ i  r ‖', '‖ oH r ‖')]
        )
        self.assertEqual(
            self.cmp.alignments('ay', 'awgeC'),
            [(20.0, '‖ a y ‖', '‖ a w ‖ geC')],
        )
        self.assertEqual(
            self.cmp.alignments('nowz', 'naHzeC'),
            [(70.5, '‖ n ow z ‖', '‖ n aH z ‖ eC')],
        )
        self.assertEqual(
            self.cmp.alignments('mawtS', 'munt'),
            [(62.5, '‖ m aw - tS ‖', '‖ m u  n t  ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('teCgN', 'tsugNeC'),
            [(75.0, '‖ t  eC gN ‖', '‖ ts u  gN ‖ eC')],
        )
        self.assertEqual(
            self.cmp.alignments('fut', 'fuHs'),
            [(74.0, '‖ f u  t ‖', '‖ f uH s ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('niy', 'kniH'),
            [(53.0, '‖ n iy ‖', 'k ‖ n iH ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('haFnd', 'hant'),
            [(107.5, '‖ h aF n d ‖', '‖ h a  n t ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('hart', 'herts'),
            [
                (115.0, '‖ h a r t ‖', '‖ h e r t ‖ s'),
                (115.0, '‖ h a r t  ‖', '‖ h e r ts ‖'),
            ],
        )
        self.assertEqual(
            self.cmp.alignments('liveCr', 'leHbeCr'),
            [(109.5, '‖ l i  v eC r ‖', '‖ l eH b eC r ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('aFnd', 'ante'),
            [(72.5, '‖ aF n d ‖', '‖ a  n t ‖ e')],
        )
        self.assertEqual(
            self.cmp.alignments('aFt', 'ad'), [(37.5, '‖ aF t ‖', '‖ a  d ‖')]
        )
        self.assertEqual(
            self.cmp.alignments('blow', 'flaHre'),
            [(52.0, '‖ b l o  ‖ w', '‖ f l aH ‖ re')],
        )
        # Different from paper:
        self.assertEqual(
            self.cmp.alignments('ir', 'awris'),
            [(45.0, '‖ i r ‖', 'a ‖ w r ‖ is')],
        )
        self.assertEqual(
            self.cmp.alignments('iyt', 'edere'),
            [(40.0, '‖ iy t ‖', '‖ e  d ‖ ere')],
        )
        self.assertEqual(
            self.cmp.alignments('fisS', 'piskis'),
            [(73.0, '‖ f i sS ‖', '‖ p i s  ‖ kis')],
        )
        self.assertEqual(
            self.cmp.alignments('flow', 'fluere'),
            [(92.5, '‖ f l ow ‖', '‖ f l u  ‖ ere')],
        )
        self.assertEqual(
            self.cmp.alignments('star', 'steHlla'),
            [(92.0, '‖ s t a  r ‖', '‖ s t eH l ‖ la')],
        )
        self.assertEqual(
            self.cmp.alignments('ful', 'pleHnus'),
            [(48.0, '‖ f u l ‖', '‖ p - l ‖ eHnus')],
        )
        self.assertEqual(
            self.cmp.alignments('graFs', 'graHmen'),
            [(81.5, '‖ g r aF ‖ s', '‖ g r aH ‖ men')],
        )
        self.assertEqual(
            self.cmp.alignments('hart', 'kordis'),
            [(70.0, '‖ h a r t ‖', '‖ k o r d ‖ is')],
        )
        self.assertEqual(
            self.cmp.alignments('horn', 'kornuH'),
            [(90.0, '‖ h o r n ‖', '‖ k o r n ‖ uH')],
        )
        self.assertEqual(
            self.cmp.alignments('ay', 'ego'), [(17.5, '‖ ay ‖', '‖ e  ‖ go')]
        )
        self.assertEqual(
            self.cmp.alignments('niy', 'genuH'),
            [(44.0, '‖ n i  ‖ y', 'ge ‖ n uH ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('meCdSeCr', 'maHter'),
            [(109.0, '‖ m eC dS eC r ‖', '‖ m aH t  e  r ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('mawnteCn', 'moHns'),
            [(105.5, '‖ m aw n t ‖ eCn', '‖ m oH n s ‖')],
        )
        # The example below is different from the expected, but
        # (73.0, '‖ n ey m ‖', '‖ n oH m ‖ en') is the #2 alignment.
        # This is probably due to slightly differing weights/costs/features.
        self.assertEqual(
            self.cmp.alignments('neym', 'noHmen'),
            [(80.5, '‖ n ey m ‖', 'noH ‖ m e  n ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('nyuw', 'nowus'),
            [(70.0, '‖ n yu w  ‖', '‖ n o  wu ‖ s')],
        )
        self.assertEqual(
            self.cmp.alignments('weCn', 'uHnus'),
            [(48.0, '‖ weC n ‖', '‖ uH  n ‖ us')],
        )
        self.assertEqual(
            self.cmp.alignments('rawnd', 'rotundus'),
            [(115.0, '‖ r a - w n d ‖', '‖ r o t u n d ‖ us')],
        )
        self.assertEqual(
            self.cmp.alignments('sow', 'suere'),
            [(57.5, '‖ s ow ‖', '‖ s u  ‖ ere')],
        )
        self.assertEqual(
            self.cmp.alignments('sit', 'seHdere'),
            [(66.5, '‖ s i  t ‖', '‖ s eH d ‖ ere')],
        )
        self.assertEqual(
            self.cmp.alignments('tSriy', 'treHs'),
            [(73.0, '‖ tS r iy ‖', '‖ t  r eH ‖ s')],
        )
        self.assertEqual(
            self.cmp.alignments('tuwtS', 'dentis'),
            [(85.0, '‖ t uw tS ‖', 'den ‖ t i  s  ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('tSin', 'tenuis'),
            [(67.5, '‖ tS i n ‖', '‖ t  e n ‖ uis')],
        )
        self.assertEqual(
            self.cmp.alignments('kiHnwaHwa', 'kenuaq'),
            [(105.5, '‖ k iH n w aH ‖ wa', '‖ k e  n u a  ‖ q')],
        )
        self.assertEqual(
            self.cmp.alignments('niHna', 'nenah'),
            [(91.5, '‖ n iH n a ‖', '‖ n e  n a ‖ h')],
        )
        self.assertEqual(
            self.cmp.alignments('naHpeHwa', 'naHpeHw'),
            [(115.0, '‖ n aH p eH w ‖ a', '‖ n aH p eH w ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('waHpimini', 'waHpemen'),
            [(150.0, '‖ w aH p i m i n ‖ i', '‖ w aH p e m e n ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('nameHsa', 'nameHqs'),
            [(125.0, '‖ n a m eH - s ‖ a', '‖ n a m eH q s ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('okimaHwa', 'okeHmaHw'),
            [(121.5, '‖ o k i  m aH w ‖ a', '‖ o k eH m aH w ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('sPiHsPiHpa', 'seHqsep'),
            [(97.0, '‖ sP iH - sP iH p ‖ a', '‖ s  eH q s  e  p ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('ahkohkwa', 'ahkeHh'),
            [(124.0, '‖ a h k o  h ‖ kwa', '‖ a h k eH h ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('pemaHtesiweni', 'pemaHtesewen'),
            [
                (
                    257.5,
                    '‖ p e m aH t e s i w e n ‖ i',
                    '‖ p e m aH t e s e w e n ‖',
                )
            ],
        )
        self.assertEqual(
            self.cmp.alignments('asenya', 'aqsen'),
            [(90.0, '‖ a - s e n ‖ ya', '‖ a q s e n ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('didoHmi', 'doH'),
            [(50.0, 'di ‖ d oH ‖ mi', '‖ d oH ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('tAugateEr', 'toxteCr'),
            [(130.0, '‖ tA u g a t e  r ‖', '‖ t  o x - t eC r ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('doteCr', 'tAugateEr'),
            [(112.5, '‖ d o t eC r ‖', 'tAu ‖ g a t e  r ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('ager', 'azPras'),
            [(61.0, '‖ a g  e r ‖', '‖ a zP - r ‖ as')],
        )
        self.assertEqual(
            self.cmp.alignments('bAaraHmi', 'pAero'),
            [(74.0, '‖ bA a r aH ‖ mi', '‖ pA e r o  ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('kentum', 'hekaton'),
            [
                (111.5, '‖ k e n t u m ‖', 'he ‖ k a - t o n ‖'),
                (111.5, '‖ k e nt u m ‖', 'he ‖ k a t  o n ‖'),
            ],
        )
        self.assertEqual(
            self.cmp.alignments('kentum', 'sateCm'),
            [
                (90.0, '‖ k e n t u  m ‖', '‖ s a - t eC m ‖'),
                (90.0, '‖ k e nt u  m ‖', '‖ s a t  eC m ‖'),
            ],
        )

        # test cases from Downey, et al. (2008)
        self.assertEqual(
            self.cmp.alignments('api', 'api'),
            [(65.0, '‖ a p i ‖', '‖ a p i ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('apik', 'apik'),
            [(100.0, '‖ a p i k ‖', '‖ a p i k ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('apila', 'apila'),
            [(115.0, '‖ a p i l a ‖', '‖ a p i l a ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('api', 'apik'),
            [(65.0, '‖ a p i ‖', '‖ a p i ‖ k')],
        )
        self.assertEqual(
            self.cmp.alignments('api', 'apila'),
            [(65.0, '‖ a p i ‖', '‖ a p i ‖ la')],
        )
        self.assertEqual(
            self.cmp.alignments('apik', 'apila'),
            [(65.0, '‖ a p i ‖ k', '‖ a p i ‖ la')],
        )
        self.assertEqual(
            self.cmp.alignments('kalarita', 'kalarita'),
            [(200.0, '‖ k a l a r i t a ‖', '‖ k a l a r i t a ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('kalara', 'kalara'),
            [(150.0, '‖ k a l a r a ‖', '‖ k a l a r a ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('makebela', 'makebela'),
            [(200.0, '‖ m a k e b e l a ‖', '‖ m a k e b e l a ‖')],
        )
        # The following case has a different score, but the same alignment as
        # in Downey, et. al (2008)
        self.assertEqual(
            self.cmp.alignments('kalarita', 'kalara'),
            [(137.5, '‖ k a l a r i ‖ ta', '‖ k a l a r a ‖')],
        )
        self.assertEqual(
            self.cmp.alignments('kalarita', 'makebela'),
            [
                (75.0, '‖ k - - a l a ‖ rita', 'ma ‖ k e b e l a ‖'),
                (75.0, '‖ k a - - l a ‖ rita', 'ma ‖ k e b e l a ‖'),
            ],
        )
        self.assertEqual(
            self.cmp.alignments('kalara', 'makebela'),
            [(82.0, '‖ k a l a r a ‖', 'ma ‖ k e b e l a ‖')],
        )

        # other alignment styles:
        cmp2 = ALINE(mode='local')
        self.assertEqual(
            cmp2.alignments('aHpakosiHs', 'waHpikonoHha'),
            [(120.0, '‖ aH p a k o s iH s ‖', 'w ‖ aH p i k o n oH h ‖ a')],
        )
        cmp2 = ALINE(mode='semi-global')
        self.assertEqual(
            cmp2.alignments('aHpakosiHs', 'waHpikonoHha'),
            [(120.0, '‖ aH p a k o s iH s ‖', 'w ‖ aH p i k o n oH h ‖ a')],
        )
        cmp2 = ALINE(mode='half-local')
        self.assertEqual(
            cmp2.alignments('aHpakosiHs', 'waHpikonoHha'),
            [(110.0, '‖ aH p a k o s iH s - ‖', 'w ‖ aH p i k o n oH h a ‖')],
        )
        cmp2 = ALINE(mode='global')
        self.assertEqual(
            cmp2.alignments('aHpakosiHs', 'waHpikonoHha'),
            [(106.5, '‖ aH  p a k o s iH s - ‖', '‖ waH p i k o n oH h a ‖')],
        )
        # The following just confirms that unknown values of mode use 'local'
        cmp2 = ALINE(mode='universal')
        self.assertEqual(
            cmp2.alignments('aHpakosiHs', 'waHpikonoHha'),
            [(120.0, '‖ aH p a k o s iH s ‖', 'w ‖ aH p i k o n oH h ‖ a')],
        )
        self.assertEqual(
            cmp2.alignments('kan', 'kaABCDHn'),
            [(84.0, '‖ k a      n ‖', '‖ k aABCDH n ‖')],
        )
        self.assertEqual(
            cmp2.alignments('kaABCDHn', 'kan'),
            [(84.0, '‖ k aABCDH n ‖', '‖ k a      n ‖')],
        )
        cmp2 = ALINE(phones='ipa')
        self.assertEqual(
            cmp2.alignments('kɒgneit', 'kognaːtus'),
            [(163.0, '‖ k ɒ g n ei t ‖', '‖ k o g n aː t ‖ us')],
        )

    def test_aline_alignment(self):
        """Test abydos.distance.ALINE.alignment."""
        self.assertEqual(
            self.cmp.alignment('ombre', 'om'), (50.0, '‖ o m ‖ bre', '‖ o m ‖')
        )
        self.assertEqual(
            self.cmp.alignment('arbol', 'arbreC'),
            (88.0, '‖ a r b o l ‖', '‖ a r b - r ‖ eC'),
        )
        self.assertEqual(
            self.cmp.alignment('pluFma', 'plum'),
            (115.0, '‖ p l uF m ‖ a', '‖ p l u  m ‖'),
        )
        self.assertEqual(
            self.cmp.alignment('kabetSa', 'kap'),
            (75.0, '‖ k a b ‖ etSa', '‖ k a p ‖'),
        )
        self.assertEqual(
            self.cmp.alignment('boka', 'busP'),
            (68.5, '‖ b o k  ‖ a', '‖ b u sP ‖'),
        )
        self.assertEqual(
            self.cmp.alignment('pye', 'pye'), (65.0, '‖ p y e ‖', '‖ p y e ‖')
        )
        self.assertEqual(
            self.cmp.alignment('koratSon', 'koFr'),
            (80.0, '‖ k o  r ‖ atSon', '‖ k oF r ‖'),
        )
        self.assertEqual(
            self.cmp.alignment('ber', 'vwar'),
            (60.5, '‖ b e  r ‖', '‖ v wa r ‖'),
        )
        self.assertEqual(
            self.cmp.alignment('benir', 'veCnir'),
            (115.5, '‖ b e  n i r ‖', '‖ v eC n i r ‖'),
        )
        self.assertEqual(
            self.cmp.alignment('detSir', 'dir'),
            (65.0, 'de ‖ tS i r ‖', '‖ d  i r ‖'),
        )

    def test_aline_sim(self):
        """Test abydos.distance.ALINE.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 1.0)
        self.assertEqual(self.cmp.sim('a', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'a'), 0.0)
        self.assertEqual(self.cmp.sim('abc', ''), 0.0)
        self.assertEqual(self.cmp.sim('', 'abc'), 0.0)
        self.assertEqual(self.cmp.sim('abc', 'abc'), 1.0)
        self.assertEqual(self.cmp.sim('abcd', 'efgh'), 0.425)

        self.assertAlmostEqual(self.cmp.sim('nigel', 'niall'), 0.7037037037)
        self.assertAlmostEqual(self.cmp.sim('niall', 'nigel'), 0.7037037037)
        self.assertAlmostEqual(self.cmp.sim('colin', 'coiln'), 0.8333333333)
        self.assertAlmostEqual(self.cmp.sim('coiln', 'colin'), 0.8333333333)
        self.assertAlmostEqual(
            self.cmp.sim('ATCAACGAGT'.lower(), 'AACGATTAG'.lower()),
            0.685185185,
        )

        # test cases from Downey, et al. (2008)
        self.assertAlmostEqual(self.cmp_downey.sim('api', 'api'), 1.0)
        self.assertAlmostEqual(self.cmp_downey.sim('apik', 'apik'), 1.0)
        self.assertAlmostEqual(self.cmp_downey.sim('apila', 'apila'), 1.0)
        self.assertAlmostEqual(
            self.cmp_downey.sim('api', 'apik'), 0.7878787879
        )
        self.assertAlmostEqual(
            self.cmp_downey.sim('api', 'apila'), 0.7222222222
        )
        self.assertAlmostEqual(
            self.cmp_downey.sim('apik', 'apila'), 0.6046511628
        )
        self.assertAlmostEqual(
            self.cmp_downey.sim('kalarita', 'kalarita'), 1.0
        )
        self.assertAlmostEqual(self.cmp_downey.sim('kalara', 'kalara'), 1.0)
        self.assertAlmostEqual(
            self.cmp_downey.sim('makebela', 'makebela'), 1.0
        )
        self.assertAlmostEqual(
            self.cmp_downey.sim('kalarita', 'kalara'), 0.785714286
        )
        self.assertAlmostEqual(
            self.cmp_downey.sim('kalarita', 'makebela'), 0.375
        )
        self.assertAlmostEqual(
            self.cmp_downey.sim('kalara', 'makebela'), 0.468571429
        )

    def test_aline_sim_score(self):
        """Test abydos.distance.ALINE.sim_score."""
        # Base cases
        self.assertEqual(self.cmp.sim_score('', ''), 1.0)
        self.assertEqual(self.cmp.sim_score('a', ''), 0.0)
        self.assertEqual(self.cmp.sim_score('', 'a'), 0.0)
        self.assertEqual(self.cmp.sim_score('abc', ''), 0.0)
        self.assertEqual(self.cmp.sim_score('', 'abc'), 0.0)
        self.assertEqual(self.cmp.sim_score('abc', 'abc'), 85.0)
        self.assertEqual(self.cmp.sim_score('abcd', 'efgh'), 51.0)

        self.assertAlmostEqual(self.cmp.sim_score('nigel', 'niall'), 95.0)
        self.assertAlmostEqual(self.cmp.sim_score('niall', 'nigel'), 95.0)
        self.assertAlmostEqual(self.cmp.sim_score('colin', 'coiln'), 112.5)
        self.assertAlmostEqual(self.cmp.sim_score('coiln', 'colin'), 112.5)
        self.assertAlmostEqual(
            self.cmp.sim_score('ATCAACGAGT'.lower(), 'AACGATTAG'.lower()),
            185.0,
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.distance.test_distance_baystat.

This module contains unit tests for abydos.distance.Baystat
"""

import unittest

from abydos.distance import Baystat


class BaystatTestCases(unittest.TestCase):
    """Test Baystat functions.

    abydos.distance.Baystat
    """

    cmp = Baystat()

    def test_baystat_sim(self):
        """Test abydos.distance.Baystat.sim."""
        # Base cases
        self.assertEqual(self.cmp.sim('', ''), 1)
        self.assertEqual(self.cmp.sim('Colin', ''), 0)
        self.assertEqual(self.cmp.sim('Colin', 'Colin'), 1)

        # Examples given in the paper
        # https://www.statistik.bayern.de/medien/statistik/zensus/zusammenf__hrung_von_datenbest__nden_ohne_numerische_identifikatoren.pdf
        self.assertAlmostEqual(self.cmp.sim('DRAKOMENA', 'DRAOMINA'), 7 / 9)
        self.assertAlmostEqual(self.cmp.sim('RIEKI', 'RILKI'), 4 / 5)
        self.assertAlmostEqual(
            self.cmp.sim('ATANASSIONI', 'ATANASIOU'), 8 / 11
        )
        self.assertAlmostEqual(
            self.cmp.sim('LIESKOVSKY', 'LIESZKOVSZKY'), 10 / 12
        )
        self.assertAlmostEqual(self.cmp.sim('JEANETTE', 'JEANNETTE'), 8 / 9)
        self.assertAlmostEqual(self.cmp.sim('JOHANNES', 'JOHAN'), 0.625)
        self.assertAlmostEqual(self.cmp.sim('JOHANNES', 'HANS'), 0.375)
        self.assertAlmostEqual(self.cmp.sim('JOHANNES', 'HANNES'), 0.75)
        self.assertAlmostEqual(self.cmp.sim('ZIMMERMANN', 'SEMMERMANN'), 0.8)
        self.assertAlmostEqual(self.cmp.sim('ZIMMERMANN', 'ZIMMERER'), 0.6)
        self.assertAlmostEqual(self.cmp.sim('ZIMMERMANN', 'ZIMMER'), 0.6)

        # Tests to maximize coverage
        self.assertAlmostEqual(
            Baystat(2, 2, 2).sim('ZIMMERMANN', 'SEMMERMANN'), 0.8
        )
        self.assertAlmostEqual(self.cmp.sim('ZIMMER', 'ZIMMERMANN'), 0.6)

    def test_baystat_dist(self):
        """Test abydos.distance.Baystat.dist."""
        # Base cases
        self.assertEqual(self.cmp.dist('', ''), 0)
        self.assertEqual(self.cmp.dist('Colin', ''), 1)
        self.assertEqual(self.cmp.dist('Colin', 'Colin'), 0)

        # Examples given in the paper
        # https://www.statistik.bayern.de/medien/statistik/zensus/zusammenf__hrung_von_datenbest__nden_ohne_numerische_identifikatoren.pdf
        self.assertAlmostEqual(self.cmp.dist('DRAKOMENA', 'DRAOMINA'), 2 / 9)
        self.assertAlmostEqual(self.cmp.dist('RIEKI', 'RILKI'), 1 / 5)
        self.assertAlmostEqual(
            self.cmp.dist('ATANASSIONI', 'ATANASIOU'), 3 / 11
        )
        self.assertAlmostEqual(
            self.cmp.dist('LIESKOVSKY', 'LIESZKOVSZKY'), 2 / 12
        )
        self.assertAlmostEqual(self.cmp.dist('JEANETTE', 'JEANNETTE'), 1 / 9)
        self.assertAlmostEqual(self.cmp.dist('JOHANNES', 'JOHAN'), 0.375)
        self.assertAlmostEqual(self.cmp.dist('JOHANNES', 'HANS'), 0.625)
        self.assertAlmostEqual(self.cmp.dist('JOHANNES', 'HANNES'), 0.25)
        self.assertAlmostEqual(self.cmp.dist('ZIMMERMANN', 'SEMMERMANN'), 0.2)
        self.assertAlmostEqual(self.cmp.dist('ZIMMERMANN', 'ZIMMERER'), 0.4)
        self.assertAlmostEqual(self.cmp.dist('ZIMMERMANN', 'ZIMMER'), 0.4)


if __name__ == '__main__':
    unittest.main()



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.tokenizer.test_tokenizer_qgrams.

This module contains unit tests for abydos.tokenizer.QGrams
"""

import unittest

from abydos.tokenizer import RegexpTokenizer


class RegexpTokenizerTestCases(unittest.TestCase):
    """Test abydos.tokenizer.RegexpTokenizer."""

    def test_regexp_tokenizer(self):
        """Test abydos.tokenizer.RegexpTokenizer."""
        self.assertEqual(sorted(RegexpTokenizer().tokenize('').get_list()), [])
        self.assertEqual(
            sorted(RegexpTokenizer().tokenize('a').get_list()), ['a']
        )

        self.assertEqual(
            sorted(RegexpTokenizer().tokenize('NELSON').get_list()),
            sorted(['NELSON']),
        )
        self.assertEqual(
            sorted(RegexpTokenizer().tokenize('NEILSEN').get_list()),
            sorted(['NEILSEN']),
        )

        tweet = "Looking forward to hearing your ideas about what we can\
        accomplish this year & beyond. I'll answer your questions on\
        #AskPOTUS at 12:30p ET."
        self.assertEqual(
            sorted(RegexpTokenizer().tokenize(tweet).get_list()),
            sorted(
                [
                    'Looking',
                    'forward',
                    'to',
                    'hearing',
                    'your',
                    'ideas',
                    'about',
                    'what',
                    'we',
                    'can',
                    'accomplish',
                    'this',
                    'year',
                    'beyond',
                    'I',
                    'll',
                    'answer',
                    'your',
                    'questions',
                    'on',
                    'AskPOTUS',
                    'at',
                    '12',
                    '30p',
                    'ET',
                ]
            ),
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""setup.py.

setuptools configuration file for Abydos
"""

from codecs import open
from os import path

from abydos import __version__

from setuptools import find_packages, setup

HERE = path.abspath(path.dirname(__file__))


def readfile(fn):
    """Read fn and return the contents.

    Parameters
    ----------
    fn : str
        A filename

    Returns
    -------
    str
        The content of the file

    """
    with open(path.join(HERE, fn), 'r', encoding='utf-8') as f:
        return f.read()


if __name__ == '__main__':
    setup(
        name='abydos',
        packages=find_packages(exclude=['tests*']),
        version=__version__,
        description='Abydos NLP/IR library',
        author='Christopher C. Little',
        author_email='chrisclittle+abydos@gmail.com',
        url='https://github.com/chrislit/abydos',
        download_url='https://github.com/chrislit/abydos/archive/master.zip',
        keywords=[
            'nlp',
            'ai',
            'ir',
            'language',
            'linguistics',
            'phonetic algorithms',
            'string distance',
        ],
        license='GPLv3+',
        classifiers=[
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'License :: OSI Approved :: GNU General Public License v3 or \
later (GPLv3+)',
            'Operating System :: OS Independent',
            'Topic :: Scientific/Engineering :: Artificial Intelligence',
            'Topic :: Software Development :: Libraries :: Python Modules',
            'Topic :: Text Processing :: Indexing',
            'Topic :: Text Processing :: Linguistic',
            'Natural Language :: English',
        ],
        long_description_content_type='text/x-rst',
        long_description='\n\n'.join(
            [readfile(f) for f in ('README.rst', 'HISTORY.rst', 'AUTHORS.rst')]
        ),
        install_requires=['numpy', 'deprecation'],
        python_requires='~=3.5',
    )



# Copyright 2019-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.fingerprint.test_fingerprint_bwtrlef.

This module contains unit tests for abydos.fingerprint.BWTRLEF
"""

import unittest

from abydos.fingerprint import BWTRLEF


class BWTRLEFTestCases(unittest.TestCase):
    """Test BWT+RLE fingerprint.

    abydos.fingerprint.BWTRLEF
    """

    bwtrle = BWTRLEF()

    bws = 'WWWWWWWWWWWWBWWWWWWWWWWWWBBBWWWWWWWWWWWWWWWWWWWWWWWWBWWWWWWWWWWWWWW'

    def test_consonant_bwtrlef(self):
        """Test abydos.fingerprint.BWTRLEF."""
        # Base case
        self.assertEqual(self.bwtrle.fingerprint(''), '\x00')

        self.assertEqual(self.bwtrle.fingerprint('banana'), 'annb\x00aa')
        self.assertEqual(
            self.bwtrle.fingerprint(self.bws), 'WWBWWB45WB\x003WB10WB'
        )
        self.assertEqual(
            self.bwtrle.fingerprint('Schifffahrt'), 't\x00fSfficahhr'
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_parmar_kumbharana.

This module contains unit tests for abydos.phonetic.ParmarKumbharana
"""

import unittest

from abydos.phonetic import ParmarKumbharana


class ParmarKumbharanaTestCases(unittest.TestCase):
    """Test Parmar-Kumbharana functions.

    test cases for abydos.phonetic.ParmarKumbharana
    """

    pa = ParmarKumbharana()

    def test_parmar_kumbharana(self):
        """Test abydos.phonetic.ParmarKumbharana."""
        # Base cases
        self.assertEqual(self.pa.encode(''), '')

        # Test cases from Parmar & Kumbharana (2014)
        test_cases = (
            ('Week', 'WK'),
            ('Weak', 'WK'),
            ('Piece', 'PS'),
            ('Peace', 'PS'),
            ('Bed', 'BD'),
            ('Bad', 'BD'),
            ('Would', 'WD'),
            ('Wood', 'WD'),
            ('Sun', 'SN'),
            ('Son', 'SN'),
            ('Ship', 'SP'),
            ('Sheep', 'SP'),
            ('Later', 'LTR'),
            ('Letter', 'LTR'),
            ('Low', 'LW'),
            ('Law', 'LW'),
            ('She', 'S'),
            ('See', 'S'),
            ('Sea', 'S'),
            ('Case', 'CS'),
            ('Cash', 'CS'),
            ('Of', 'OF'),
            ('Off', 'OF'),
            ('Live', 'LV'),
            ('Leave', 'LV'),
            ('Sign', 'SN'),
            ('Sine', 'SN'),
            ('Sin', 'SN'),
            ('Seen', 'SN'),
            ('By', 'B'),
            ('Bye', 'B'),
            ('Reach', 'RCH'),
            ('Rich', 'RCH'),
            ('Sort', 'SRT'),
            ('Short', 'SRT'),
            ('Center', 'SNTR'),
            ('Centre', 'SNTR'),
            ('Full', 'FL'),
            ('Fool', 'FL'),
            ('Then', 'THN'),
            ('Than', 'THN'),
            ('Fill', 'FL'),
            ('Feel', 'FL'),
            ('Two', 'TW'),
            ('To', 'T'),
            ('Too', 'T'),
            ('Four', 'FR'),
            ('For', 'FR'),
            ('Mat', 'MT'),
            ('Met', 'MT'),
            ('Merry', 'MR'),
            ('Marry', 'MR'),
        )
        for word, encoding in test_cases:
            self.assertEqual(self.pa.encode(word), encoding)


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_snowball_german.

This module contains unit tests for abydos.stemmer.SnowballGerman
"""

import codecs
import unittest

from abydos.stemmer import SnowballGerman

from .. import _corpus_file


class SnowballGermanTestCases(unittest.TestCase):
    """Test Snowball German functions.

    abydos.stemmer.SnowballGerman
    """

    stmr = SnowballGerman()
    stmr_av = SnowballGerman(alternate_vowels=True)

    def test_snowball_german(self):
        """Test abydos.stemmer.SnowballGerman (Snowball testset).

        These test cases are from
        http://snowball.tartarus.org/algorithms/german/diffs.txt
        """
        # base case
        self.assertEqual(self.stmr.stem(''), '')

        #  Snowball German test set
        with codecs.open(
            _corpus_file('snowball_german.csv'), encoding='utf-8'
        ) as snowball_ts:
            next(snowball_ts)
            for line in snowball_ts:
                if line[0] != '#':
                    line = line.strip().split(',')
                    word, stem = line[0], line[1]
                    self.assertEqual(self.stmr.stem(word), stem.lower())

        # missed branch test cases
        self.assertEqual(self.stmr.stem('ikeit'), 'ikeit')

    def test_sb_german_snowball_alt(self):
        """Test abydos.stemmer.SnowballGerman (alternate vowels)."""
        # base case
        self.assertEqual(self.stmr_av.stem(''), '')

        # dämmerung,dammer
        self.assertEqual(self.stmr_av.stem('dämmerung'), 'dammer')
        self.assertEqual(self.stmr_av.stem('daemmerung'), 'dammer')
        self.assertEqual(self.stmr.stem('dämmerung'), 'dammer')
        self.assertEqual(self.stmr.stem('daemmerung'), 'daemmer')

        # brötchen,brotch
        self.assertEqual(self.stmr_av.stem('brötchen'), 'brotch')
        self.assertEqual(self.stmr_av.stem('broetchen'), 'brotch')
        self.assertEqual(self.stmr.stem('brötchen'), 'brotch')
        self.assertEqual(self.stmr.stem('broetchen'), 'broetch')

        # büro,buro
        self.assertEqual(self.stmr_av.stem('büro'), 'buro')
        self.assertEqual(self.stmr_av.stem('buero'), 'buro')
        self.assertEqual(self.stmr.stem('büro'), 'buro')
        self.assertEqual(self.stmr.stem('buero'), 'buero')

        # häufen,hauf
        self.assertEqual(self.stmr_av.stem('häufen'), 'hauf')
        self.assertEqual(self.stmr_av.stem('haeufen'), 'hauf')
        self.assertEqual(self.stmr.stem('häufen'), 'hauf')
        self.assertEqual(self.stmr.stem('haeufen'), 'haeuf')

        # quelle,quell
        self.assertEqual(self.stmr_av.stem('qülle'), 'qull')
        self.assertEqual(self.stmr_av.stem('quelle'), 'quell')
        self.assertEqual(self.stmr.stem('qülle'), 'qull')
        self.assertEqual(self.stmr.stem('quelle'), 'quell')

        # feuer,feuer
        self.assertEqual(self.stmr_av.stem('feür'), 'feur')
        self.assertEqual(self.stmr_av.stem('feuer'), 'feu')
        self.assertEqual(self.stmr.stem('feür'), 'feur')
        self.assertEqual(self.stmr.stem('feuer'), 'feu')

        # über,uber
        self.assertEqual(self.stmr_av.stem('über'), 'uber')
        self.assertEqual(self.stmr_av.stem('ueber'), 'uber')
        self.assertEqual(self.stmr.stem('über'), 'uber')
        self.assertEqual(self.stmr.stem('ueber'), 'ueb')


if __name__ == '__main__':
    unittest.main()



# Copyright 2014-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.phonetic.test_phonetic_beider_morse.

This module contains unit tests for abydos.phonetic.BeiderMorse
"""

import codecs
import unittest

from abydos.phonetic import BeiderMorse

# noinspection PyProtectedMember
from abydos.phonetic._beider_morse_data import (
    L_ANY,
    L_CYRILLIC,
    L_CZECH,
    L_DUTCH,
    L_ENGLISH,
    L_FRENCH,
    L_GERMAN,
    L_GREEK,
    L_GREEKLATIN,
    L_HEBREW,
    L_HUNGARIAN,
    L_ITALIAN,
    L_LATVIAN,
    L_POLISH,
    L_PORTUGUESE,
    L_ROMANIAN,
    L_SPANISH,
    L_TURKISH,
)

from .. import ALLOW_RANDOM, _corpus_file, _one_in


class BeiderMorseTestCases(unittest.TestCase):
    """Test BeiderMorse functions.

    test cases for abydos.phonetic.BeiderMorse
    """

    pa = BeiderMorse()

    def test_beider_morse_encode(self):
        """Test abydos.phonetic.BeiderMorse.

        Most test cases from:
        http://svn.apache.org/viewvc/commons/proper/codec/trunk/src/test/java/org/apache/commons/codec/language/bm/

        As a rule, the test cases are copied from the above code, but the
        resultant values are not. This is largely because this Python port
        follows the PHP reference implementation much more closely than the
        Java port in Apache Commons Codec does. As a result, these tests have
        been conformed to the output produced by the PHP implementation,
        particularly in terms of formatting and ordering.
        """
        # base cases
        self.assertEqual(BeiderMorse().encode(''), '')

        for langs in ('', 1, 'spanish', 'english,italian', 3):
            for name_mode in ('gen', 'ash', 'sep'):
                for match_mode in ('approx', 'exact'):
                    for concat in (False, True):
                        if isinstance(langs, str) and (
                            (name_mode == 'ash' and 'italian' in langs)
                            or (name_mode == 'sep' and 'english' in langs)
                        ):
                            self.assertRaises(
                                ValueError,
                                BeiderMorse,
                                langs,
                                name_mode,
                                match_mode,
                                concat,
                            )
                        else:
                            self.assertEqual(
                                BeiderMorse(
                                    langs, name_mode, match_mode, concat
                                ).encode(''),
                                '',
                            )

        # testSolrGENERIC
        # concat is true, ruleType is EXACT
        self.assertEqual(
            BeiderMorse('', 'gen', 'exact', True).encode('Angelo'),
            'angelo,anxelo,anhelo,anjelo,anZelo,andZelo',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'exact', True).encode("D'Angelo"),
            'angelo,anxelo,anhelo,anjelo,anZelo,andZelo,dangelo'
            + ',danxelo,danhelo,danjelo,danZelo,dandZelo',
        )
        self.assertEqual(
            BeiderMorse('italian,greek,spanish', 'gen', 'exact', True).encode(
                'Angelo'
            ),
            'angelo,anxelo,andZelo',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'exact', True).encode('1234'), ''
        )

        # concat is false, ruleType is EXACT
        self.assertEqual(
            BeiderMorse('', 'gen', 'exact', False).encode('Angelo'),
            'angelo,anxelo,anhelo,anjelo,anZelo,andZelo',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'exact', False).encode("D'Angelo"),
            'angelo,anxelo,anhelo,anjelo,anZelo,andZelo,dangelo'
            + ',danxelo,danhelo,danjelo,danZelo,dandZelo',
        )
        self.assertEqual(
            BeiderMorse('italian,greek,spanish', 'gen', 'exact', False).encode(
                'Angelo'
            ),
            'angelo,anxelo,andZelo',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'exact', False).encode('1234'), ''
        )

        # concat is true, ruleType is APPROX
        self.assertEqual(
            BeiderMorse('', 'gen', 'approx', True).encode('Angelo'),
            'angilo,angYlo,agilo,ongilo,ongYlo,ogilo,Yngilo'
            + ',YngYlo,anxilo,onxilo,anilo,onilo,aniilo,oniilo'
            + ',anzilo,onzilo',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'approx', True).encode("D'Angelo"),
            'angilo,angYlo,agilo,ongilo,ongYlo,ogilo,Yngilo'
            + ',YngYlo,anxilo,onxilo,anilo,onilo,aniilo,oniilo'
            + ',anzilo,onzilo,dangilo,dangYlo,dagilo,dongilo'
            + ',dongYlo,dogilo,dYngilo,dYngYlo,danxilo,donxilo'
            + ',danilo,donilo,daniilo,doniilo,danzilo,donzilo',
        )
        self.assertEqual(
            BeiderMorse('italian,greek,spanish', 'gen', 'approx', True).encode(
                'Angelo'
            ),
            'angilo,ongilo,anxilo,onxilo,anzilo,onzilo',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'approx', True).encode('1234'), ''
        )

        # concat is false, ruleType is APPROX
        self.assertEqual(
            BeiderMorse('', 'gen', 'approx', False).encode('Angelo'),
            'angilo,angYlo,agilo,ongilo,ongYlo,ogilo,Yngilo'
            + ',YngYlo,anxilo,onxilo,anilo,onilo,aniilo,oniilo'
            + ',anzilo,onzilo',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'approx', False).encode("D'Angelo"),
            'angilo,angYlo,agilo,ongilo,ongYlo,ogilo,Yngilo'
            + ',YngYlo,anxilo,onxilo,anilo,onilo,aniilo,oniilo'
            + ',anzilo,onzilo,dangilo,dangYlo,dagilo,dongilo'
            + ',dongYlo,dogilo,dYngilo,dYngYlo,danxilo,donxilo'
            + ',danilo,donilo,daniilo,doniilo,danzilo,donzilo',
        )
        self.assertEqual(
            BeiderMorse(
                'italian,greek,spanish', 'gen', 'approx', False
            ).encode('Angelo'),
            'angilo,ongilo,anxilo,onxilo,anzilo,onzilo',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'approx', False).encode('1234'), ''
        )

        # testSolrASHKENAZI
        # concat is true, ruleType is EXACT
        self.assertEqual(
            BeiderMorse('', 'ash', 'exact', True).encode('Angelo'),
            'angelo,andZelo,anhelo,anxelo',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'exact', True).encode("D'Angelo"),
            'dangelo,dandZelo,danhelo,danxelo',
        )
        self.assertRaises(
            ValueError,
            BeiderMorse,
            'italian,greek,spanish',
            'ash',
            'exact',
            True,
        )
        self.assertEqual(
            BeiderMorse(
                'italian,greek,spanish', 'ash', 'exact', True, True
            ).encode('Angelo'),
            'anxelo,angelo',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'exact', True).encode('1234'), ''
        )

        # concat is false, ruleType is EXACT
        self.assertEqual(
            BeiderMorse('', 'ash', 'exact', False).encode('Angelo'),
            'angelo,andZelo,anhelo,anxelo',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'exact', False).encode("D'Angelo"),
            'dangelo,dandZelo,danhelo,danxelo',
        )
        self.assertRaises(
            ValueError,
            BeiderMorse,
            'italian,greek,spanish',
            'ash',
            'exact',
            False,
        )
        self.assertEqual(
            BeiderMorse(
                'italian,greek,spanish', 'ash', 'exact', False, True
            ).encode('Angelo'),
            'anxelo,angelo',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'exact', False).encode('1234'), ''
        )

        # concat is true, ruleType is APPROX
        self.assertEqual(
            BeiderMorse('', 'ash', 'approx', True).encode('Angelo'),
            'angilo,angYlo,ongilo,ongYlo,Yngilo,YngYlo,anzilo'
            + ',onzilo,anilo,onilo,anxilo,onxilo',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'approx', True).encode("D'Angelo"),
            'dangilo,dangYlo,dongilo,dongYlo,dYngilo,dYngYlo'
            + ',danzilo,donzilo,danilo,donilo,danxilo,donxilo',
        )
        self.assertRaises(
            ValueError,
            BeiderMorse,
            'italian,greek,spanish',
            'ash',
            'approx',
            True,
        )
        self.assertEqual(
            BeiderMorse(
                'italian,greek,spanish', 'ash', 'approx', True, True
            ).encode('Angelo'),
            'anxYlo,anxilo,onxYlo,onxilo,angYlo,angilo,ongYlo,ongilo',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'approx', True).encode('1234'), ''
        )

        # concat is false, ruleType is APPROX
        self.assertEqual(
            BeiderMorse('', 'ash', 'approx', False).encode('Angelo'),
            'angilo,angYlo,ongilo,ongYlo,Yngilo,YngYlo,anzilo'
            + ',onzilo,anilo,onilo,anxilo,onxilo',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'approx', False).encode("D'Angelo"),
            'dangilo,dangYlo,dongilo,dongYlo,dYngilo,dYngYlo'
            + ',danzilo,donzilo,danilo,donilo,danxilo,donxilo',
        )
        self.assertRaises(
            ValueError,
            BeiderMorse,
            'italian,greek,spanish',
            'ash',
            'approx',
            False,
        )
        self.assertEqual(
            BeiderMorse(
                'italian,greek,spanish', 'ash', 'approx', False, True
            ).encode('Angelo'),
            'anxYlo,anxilo,onxYlo,onxilo,angYlo,angilo,ongYlo,ongilo',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'approx', False).encode('1234'), ''
        )

        # testSolrSEPHARDIC
        # concat is true, ruleType is EXACT
        self.assertEqual(
            BeiderMorse('', 'sep', 'exact', True).encode('Angelo'),
            'anZelo,andZelo,anxelo',
        )
        self.assertEqual(
            BeiderMorse('', 'sep', 'exact', True).encode("D'Angelo"),
            'anZelo,andZelo,anxelo',
        )
        self.assertRaises(
            ValueError,
            BeiderMorse,
            'italian,greek,spanish',
            'sep',
            'exact',
            True,
        )
        self.assertEqual(
            BeiderMorse(
                'italian,greek,spanish', 'sep', 'exact', True, True
            ).encode('Angelo'),
            'andZelo,anxelo',
        )
        self.assertEqual(
            BeiderMorse('', 'sep', 'exact', True).encode('1234'), ''
        )

        # concat is false, ruleType is EXACT
        self.assertEqual(
            BeiderMorse('', 'sep', 'exact', False).encode('Angelo'),
            'anZelo,andZelo,anxelo',
        )
        self.assertEqual(
            BeiderMorse('', 'sep', 'exact', False).encode("D'Angelo"),
            'anZelo,andZelo,anxelo',
        )
        self.assertRaises(
            ValueError,
            BeiderMorse,
            'italian,greek,spanish',
            'sep',
            'exact',
            False,
        )
        self.assertEqual(
            BeiderMorse(
                'italian,greek,spanish', 'sep', 'exact', False, True
            ).encode('Angelo'),
            'andZelo,anxelo',
        )
        self.assertEqual(
            BeiderMorse('', 'sep', 'exact', False).encode('1234'), ''
        )

        # concat is true, ruleType is APPROX
        self.assertEqual(
            BeiderMorse('', 'sep', 'approx', True).encode('Angelo'),
            'anzila,anzilu,nzila,nzilu,anhila,anhilu,nhila,nhilu',
        )
        self.assertEqual(
            BeiderMorse('', 'sep', 'approx', True).encode("D'Angelo"),
            'anzila,anzilu,nzila,nzilu,anhila,anhilu,nhila,nhilu',
        )
        self.assertRaises(
            ValueError,
            BeiderMorse,
            'italian,greek,spanish',
            'sep',
            'approx',
            True,
        )
        self.assertEqual(
            BeiderMorse(
                'italian,greek,spanish', 'sep', 'approx', True, True
            ).encode('Angelo'),
            'anzila,anzilu,nzila,nzilu,anhila,anhilu,nhila,nhilu',
        )
        self.assertEqual(
            BeiderMorse('', 'sep', 'approx', True).encode('1234'), ''
        )

        # concat is false, ruleType is APPROX
        self.assertEqual(
            BeiderMorse('', 'sep', 'approx', False).encode('Angelo'),
            'anzila,anzilu,nzila,nzilu,anhila,anhilu,nhila,nhilu',
        )
        self.assertEqual(
            BeiderMorse('', 'sep', 'approx', False).encode("D'Angelo"),
            'anzila,anzilu,nzila,nzilu,anhila,anhilu,nhila,nhilu',
        )
        self.assertRaises(
            ValueError,
            BeiderMorse,
            'italian,greek,spanish',
            'sep',
            'approx',
            False,
        )
        self.assertEqual(
            BeiderMorse(
                'italian,greek,spanish', 'sep', 'approx', False, True
            ).encode('Angelo'),
            'anzila,anzilu,nzila,nzilu,anhila,anhilu,nhila,nhilu',
        )
        self.assertEqual(
            BeiderMorse('', 'sep', 'approx', False).encode('1234'), ''
        )

        # testCompatibilityWithOriginalVersion
        self.assertEqual(
            BeiderMorse('', 'gen', 'approx', False).encode('abram'),
            'abram,abrom,avram,avrom,obram,obrom,ovram,ovrom'
            + ',Ybram,Ybrom,abran,abron,obran,obron',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'approx', False).encode('Bendzin'),
            'binzn,bindzn,vindzn,bintsn,vintsn',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'approx', False).encode('abram'),
            'abram,abrom,avram,avrom,obram,obrom,ovram,ovrom'
            + ',Ybram,Ybrom,ombram,ombrom,imbram,imbrom',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'approx', False).encode('Halpern'),
            'alpirn,alpYrn,olpirn,olpYrn,Ylpirn,YlpYrn,xalpirn,xolpirn',
        )

        # PhoneticEngineTest
        self.assertEqual(
            BeiderMorse('', 'gen', 'approx', True).encode('Renault'),
            'rinolt,rino,rinDlt,rinalt,rinult,rinD,rina,rinu',
        )
        self.assertEqual(
            BeiderMorse('', 'ash', 'approx', True).encode('Renault'),
            'rinDlt,rinalt,rinult,rYnDlt,rYnalt,rYnult,rinolt',
        )
        self.assertEqual(
            BeiderMorse('', 'sep', 'approx', True).encode('Renault'), 'rinDlt'
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'exact', True).encode('SntJohn-Smith'),
            'sntjonsmit',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'exact', True).encode("d'ortley"),
            'ortlaj,ortlej,dortlaj,dortlej',
        )
        self.assertEqual(
            BeiderMorse('', 'gen', 'exact', False).encode('van helsing'),
            'helSink,helsink,helzink,xelsink,elSink,elsink'
            + ',vanhelsink,vanhelzink,vanjelsink,fanhelsink'
            + ',fanhelzink,banhelsink',
        )

    def test_beider_morse_encode_misc(self):
        """Test abydos.phonetic.BeiderMorse (miscellaneous tests).

        The purpose of this test set is to achieve higher code coverage
        and to hit some of the test cases noted in the BMPM reference code.
        """
        # test of Ashkenazi with discardable prefix
        self.assertEqual(
            BeiderMorse(name_mode='ash').encode('bar Hayim'), 'Dm,xDm'
        )

        # tests of concat behavior
        self.assertEqual(
            BeiderMorse(concat=False).encode('Rodham Clinton'),
            'rodam,rodom,rYdam,rYdom,rodan,rodon,rodxam,rodxom'
            + ',rodxan,rodxon,rudam,rudom,klinton,klnton,klintun'
            + ',klntun,tzlinton,tzlnton,tzlintun,tzlntun,zlinton'
            + ',zlnton',
        )
        self.assertEqual(
            BeiderMorse(concat=True).encode('Rodham Clinton'),
            'rodamklinton,rodomklinton,rodamklnton,rodomklnton'
            + ',rodamklintun,rodomklintun,rodamklntun,rodomklntun'
            + ',rodamtzlinton,rodomtzlinton,rodamtzlnton'
            + ',rodomtzlnton,rodamtzlintun,rodomtzlintun'
            + ',rodamtzlntun,rodomtzlntun,rodamzlinton'
            + ',rodomzlinton,rodamzlnton,rodomzlnton,rodanklinton'
            + ',rodonklinton,rodanklnton,rodonklnton'
            + ',rodxamklinton,rodxomklinton,rodxamklnton'
            + ',rodxomklnton,rodxanklinton,rodxonklinton'
            + ',rodxanklnton,rodxonklnton,rudamklinton'
            + ',rudomklinton,rudamklnton,rudomklnton,rudamklintun'
            + ',rudomklintun,rudamklntun,rudomklntun'
            + ',rudamtzlinton,rudomtzlinton,rudamtzlnton'
            + ',rudomtzlnton,rudamtzlintun,rudomtzlintun'
            + ',rudamtzlntun,rudomtzlntun',
        )

        # tests of name_mode values
        self.assertEqual(
            BeiderMorse(name_mode='ash').encode('bar Hayim'), 'Dm,xDm'
        )
        self.assertEqual(
            BeiderMorse(name_mode='ashkenazi').encode('bar Hayim'), 'Dm,xDm'
        )
        self.assertEqual(
            BeiderMorse(name_mode='Ashkenazi').encode('bar Hayim'), 'Dm,xDm'
        )
        self.assertEqual(
            BeiderMorse(name_mode='gen', concat=True).encode('bar Hayim'),
            'barDm,borDm,bYrDm,varDm,vorDm,barDn,borDn,barxDm'
            + ',borxDm,varxDm,vorxDm,barxDn,borxDn',
        )
        self.assertEqual(
            BeiderMorse(name_mode='general', concat=True).encode('bar Hayim'),
            'barDm,borDm,bYrDm,varDm,vorDm,barDn,borDn,barxDm'
            + ',borxDm,varxDm,vorxDm,barxDn,borxDn',
        )
        self.assertEqual(
            BeiderMorse(name_mode='Mizrahi', concat=True).encode('bar Hayim'),
            'barDm,borDm,bYrDm,varDm,vorDm,barDn,borDn,barxDm'
            + ',borxDm,varxDm,vorxDm,barxDn,borxDn',
        )
        self.assertEqual(
            BeiderMorse(name_mode='mizrahi', concat=True).encode('bar Hayim'),
            'barDm,borDm,bYrDm,varDm,vorDm,barDn,borDn,barxDm'
            + ',borxDm,varxDm,vorxDm,barxDn,borxDn',
        )
        self.assertEqual(
            BeiderMorse(name_mode='miz', concat=True).encode('bar Hayim'),
            'barDm,borDm,bYrDm,varDm,vorDm,barDn,borDn,barxDm'
            + ',borxDm,varxDm,vorxDm,barxDn,borxDn',
        )

        # test that out-of-range language_arg results in L_ANY
        self.assertEqual(
            BeiderMorse(language_arg=2 ** 32).encode('Rodham Clinton'),
            'rodam,rodom,rYdam,rYdom,rodan,rodon,rodxam,rodxom'
            + ',rodxan,rodxon,rudam,rudom,klinton,klnton,klintun'
            + ',klntun,tzlinton,tzlnton,tzlintun,tzlntun,zlinton'
            + ',zlnton',
        )
        self.assertEqual(
            BeiderMorse(language_arg=-4).encode('Rodham Clinton'),
            'rodam,rodom,rYdam,rYdom,rodan,rodon,rodxam,rodxom'
            + ',rodxan,rodxon,rudam,rudom,klinton,klnton,klintun'
            + ',klntun,tzlinton,tzlnton,tzlintun,tzlntun,zlinton'
            + ',zlnton',
        )

        # etc. (for code coverage)
        self.assertEqual(
            BeiderMorse(name_mode='sep').encode('van Damme'), 'dami,mi,dam,m'
        )

    def test_beider_morse_encode_nachnamen(self):
        """Test abydos.phonetic.BeiderMorse (Nachnamen set)."""
        if not ALLOW_RANDOM:
            return
        with codecs.open(
            _corpus_file('nachnamen.bm.csv'), encoding='utf-8'
        ) as nachnamen_testset:
            next(nachnamen_testset)
            for nn_line in nachnamen_testset:
                nn_line = nn_line.strip().split(',')
                # This test set is very large (~10000 entries)
                # so let's just randomly select about 20 for testing
                if nn_line[0] != '#' and _one_in(500):
                    self.assertEqual(
                        BeiderMorse(language_arg='german').encode(nn_line[0]),
                        ','.join(nn_line[1].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse().encode(nn_line[0]),
                        ','.join(nn_line[2].split(' ')),
                    )

    def test_beider_morse_encode_nachnamen_cc(self):
        """Test abydos.phonetic.BeiderMorse (Nachnamen, corner cases)."""
        with codecs.open(
            _corpus_file('nachnamen.bm.cc.csv'), encoding='utf-8'
        ) as nachnamen_testset:
            next(nachnamen_testset)
            for nn_line in nachnamen_testset:
                nn_line = nn_line.strip().split(',')
                # This test set is very large (~10000 entries)
                # so let's just randomly select about 20 for testing
                if nn_line[0] != '#':
                    self.assertEqual(
                        BeiderMorse(language_arg='german').encode(nn_line[0]),
                        ','.join(nn_line[1].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse().encode(nn_line[0]),
                        ','.join(nn_line[2].split(' ')),
                    )

    def test_beider_morse_encode_uscensus2000(self):
        """Test abydos.phonetic.BeiderMorse (US Census 2000 set)."""
        if not ALLOW_RANDOM:
            return
        with open(_corpus_file('uscensus2000.bm.csv')) as uscensus_ts:
            next(uscensus_ts)
            for cen_line in uscensus_ts:
                cen_line = cen_line.strip().split(',')
                # This test set is very large (~150000 entries)
                # so let's just randomly select about 20 for testing
                if cen_line[0] != '#' and _one_in(7500):
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='approx', name_mode='gen'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[1].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='approx', name_mode='ash'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[2].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='approx', name_mode='sep'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[3].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='exact', name_mode='gen'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[4].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='exact', name_mode='ash'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[5].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='exact', name_mode='sep'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[6].split(' ')),
                    )

    def test_beider_morse_encode_uscensus2000_cc(self):
        """Test abydos.phonetic.BeiderMorse (US Census 2000, corner cases)."""
        with open(_corpus_file('uscensus2000.bm.cc.csv')) as uscensus_ts:
            next(uscensus_ts)
            for cen_line in uscensus_ts:
                cen_line = cen_line.strip().split(',')
                # This test set is very large (~150000 entries)
                # so let's just randomly select about 20 for testing
                if cen_line[0] != '#' and _one_in(10):
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='approx', name_mode='gen'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[1].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='approx', name_mode='ash'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[2].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='approx', name_mode='sep'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[3].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='exact', name_mode='gen'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[4].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='exact', name_mode='ash'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[5].split(' ')),
                    )
                    self.assertEqual(
                        BeiderMorse(
                            match_mode='exact', name_mode='sep'
                        ).encode(cen_line[0]),
                        ','.join(cen_line[6].split(' ')),
                    )

    def test_beider_morse_phonetic_number(self):
        """Test abydos.phonetic.BeiderMorse._phonetic_number."""
        self.assertEqual(self.pa._phonetic_number(''), '')  # noqa: SF01
        self.assertEqual(
            self.pa._phonetic_number('abcd'), 'abcd'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._phonetic_number('abcd[123]'), 'abcd'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._phonetic_number('abcd[123'), 'abcd'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._phonetic_number('abcd['), 'abcd'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._phonetic_number('abcd[[[123]]]'), 'abcd'  # noqa: SF01
        )

    def test_beider_morse_apply_rule_if_compat(self):
        """Test abydos.phonetic.BeiderMorse._apply_rule_if_compat."""
        self.assertEqual(
            self.pa._apply_rule_if_compat('abc', 'def', 4),  # noqa: SF01
            'abcdef',
        )
        self.assertEqual(
            self.pa._apply_rule_if_compat('abc', 'def[6]', 4),  # noqa: SF01
            'abcdef[4]',
        )
        self.assertEqual(
            self.pa._apply_rule_if_compat('abc', 'def[4]', 4),  # noqa: SF01
            'abcdef[4]',
        )
        self.assertEqual(
            self.pa._apply_rule_if_compat('abc', 'def[0]', 4),  # noqa: SF01
            None,
        )
        self.assertEqual(
            self.pa._apply_rule_if_compat('abc', 'def[8]', 4),  # noqa: SF01
            None,
        )
        self.assertEqual(
            self.pa._apply_rule_if_compat('abc', 'def', 1),  # noqa: SF01
            'abcdef',
        )
        self.assertEqual(
            self.pa._apply_rule_if_compat('abc', 'def[4]', 1),  # noqa: SF01
            'abcdef[4]',
        )

    def test_beider_morse_language(self):
        """Test abydos.phonetic.BeiderMorse._language.

        Most test cases from:
        http://svn.apache.org/viewvc/commons/proper/codec/trunk/src/test/java/org/apache/commons/codec/language/bm/LanguageGuessingTest.java?view=markup
        """
        self.assertEqual(
            self.pa._language('Renault', 'gen'), L_FRENCH  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Mickiewicz', 'gen'), L_POLISH  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Thompson', 'gen') & L_ENGLISH,  # noqa: SF01
            L_ENGLISH,
        )
        self.assertEqual(
            self.pa._language('Nuñez', 'gen'), L_SPANISH  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Carvalho', 'gen'), L_PORTUGUESE  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Čapek', 'gen'),  # noqa: SF01
            L_CZECH | L_LATVIAN,
        )
        self.assertEqual(
            self.pa._language('Sjneijder', 'gen'), L_DUTCH  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Klausewitz', 'gen'), L_GERMAN  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Küçük', 'gen'), L_TURKISH  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Giacometti', 'gen'), L_ITALIAN  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Nagy', 'gen'), L_HUNGARIAN  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Ceauşescu', 'gen'), L_ROMANIAN  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Angelopoulos', 'gen'),  # noqa: SF01
            L_GREEKLATIN,
        )
        self.assertEqual(
            self.pa._language('Αγγελόπουλος', 'gen'), L_GREEK  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('Пушкин', 'gen'), L_CYRILLIC  # noqa: SF01
        )
        self.assertEqual(
            self.pa._language('כהן', 'gen'), L_HEBREW  # noqa: SF01
        )
        self.assertEqual(self.pa._language('ácz', 'gen'), L_ANY)  # noqa: SF01
        self.assertEqual(self.pa._language('átz', 'gen'), L_ANY)  # noqa: SF01

    def test_beider_morse_expand_alternates(self):
        """Test abydos.phonetic.BeiderMorse._expand_alternates."""
        self.assertEqual(self.pa._expand_alternates(''), '')  # noqa: SF01
        self.assertEqual(self.pa._expand_alternates('aa'), 'aa')  # noqa: SF01
        self.assertEqual(
            self.pa._expand_alternates('aa|bb'), 'aa|bb'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._expand_alternates('aa|aa'), 'aa|aa'  # noqa: SF01
        )

        self.assertEqual(
            self.pa._expand_alternates('(aa)(bb)'), 'aabb'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._expand_alternates('(aa)(bb[0])'), ''  # noqa: SF01
        )
        self.assertEqual(
            self.pa._expand_alternates('(aa)(bb[4])'), 'aabb[4]'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._expand_alternates('(aa[0])(bb)'), ''  # noqa: SF01
        )
        self.assertEqual(
            self.pa._expand_alternates('(aa[4])(bb)'), 'aabb[4]'  # noqa: SF01
        )

        self.assertEqual(
            self.pa._expand_alternates('(a|b|c)(a|b|c)'),  # noqa: SF01
            'aa|ab|ac|ba|bb|bc|ca|cb|cc',
        )
        self.assertEqual(
            self.pa._expand_alternates('(a[1]|b[2])(c|d)'),  # noqa: SF01
            'ac[1]|ad[1]|bc[2]|bd[2]',
        )
        self.assertEqual(
            self.pa._expand_alternates('(a[1]|b[2])(c[4]|d)'),  # noqa: SF01
            'ad[1]|bd[2]',
        )

    def test_beider_morse_remove_dupes(self):
        """Test abydos.phonetic.BeiderMorse._remove_dupes."""
        self.assertEqual(self.pa._remove_dupes(''), '')  # noqa: SF01
        self.assertEqual(self.pa._remove_dupes('aa'), 'aa')  # noqa: SF01
        self.assertEqual(self.pa._remove_dupes('aa|bb'), 'aa|bb')  # noqa: SF01
        self.assertEqual(self.pa._remove_dupes('aa|aa'), 'aa')  # noqa: SF01
        self.assertEqual(
            self.pa._remove_dupes('aa|aa|aa|bb|aa'), 'aa|bb'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._remove_dupes('bb|aa|bb|aa|bb'), 'bb|aa'  # noqa: SF01
        )

    def test_beider_morse_normalize_lang_attrs(self):
        """Test abydos.phonetic.BeiderMorse._normalize_language_attributes."""
        self.assertEqual(
            self.pa._normalize_lang_attrs('', False), ''  # noqa: SF01
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('', True), ''  # noqa: SF01
        )

        self.assertRaises(
            ValueError,
            self.pa._normalize_lang_attrs,  # noqa: SF01
            'a[1',
            False,
        )
        self.assertRaises(
            ValueError,
            self.pa._normalize_lang_attrs,  # noqa: SF01
            'a[1',
            True,
        )

        self.assertEqual(
            self.pa._normalize_lang_attrs('abc', False), 'abc'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('abc[0]', False), '[0]'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('abc[2]', False),  # noqa: SF01
            'abc[2]',
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('abc[2][4]', False),  # noqa: SF01
            '[0]',
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('abc[2][6]', False),  # noqa: SF01
            'abc[2]',
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('ab[2]c[4]', False),  # noqa: SF01
            '[0]',
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('ab[2]c[6]', False),  # noqa: SF01
            'abc[2]',
        )

        self.assertEqual(
            self.pa._normalize_lang_attrs('abc', True), 'abc'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('abc[0]', True), 'abc'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('abc[2]', True), 'abc'  # noqa: SF01
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('abc[2][4]', True),  # noqa: SF01
            'abc',
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('abc[2][6]', True),  # noqa: SF01
            'abc',
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('ab[2]c[4]', True),  # noqa: SF01
            'abc',
        )
        self.assertEqual(
            self.pa._normalize_lang_attrs('ab[2]c[6]', True),  # noqa: SF01
            'abc',
        )

        self.assertEqual(
            self.pa._language_index_from_code(0, 'gen'), L_ANY  # noqa: SF01
        )


if __name__ == '__main__':
    unittest.main()



# Copyright 2018-2020 by Christopher C. Little.
# This file is part of Abydos.
#
# Abydos is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Abydos is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Abydos. If not, see <http://www.gnu.org/licenses/>.

"""abydos.tests.stemmer.test_stemmer_schinke.

This module contains unit tests for abydos.stemmer.Schinke
"""

import unittest

from abydos.stemmer import Schinke

from .. import _corpus_file


class SchinkeTestCases(unittest.TestCase):
    """Test Schinke functions.

    abydos.stemmer.Schinke
    """

    stmr = Schinke()

    def test_schinke(self):
        """Test abydos.stemmer.Schinke.

        These tests are copied from the Snowball testset at
        http://snowball.tartarus.org/otherapps/schinke/schinke.tgz
        """
        with open(_corpus_file('snowball_schinke.csv')) as schinke_ts:
            for schinke_line in schinke_ts:
                word, noun, verb = schinke_line.strip().split(',')
                nv = self.stmr.stem_dict(word)
                self.assertEqual(nv['n'], noun)
                self.assertEqual(nv['v'], verb)
                nv = self.stmr.stem(word)
                self.assertEqual(nv, '{},{}'.format(noun, verb))


if __name__ == '__main__':
    unittest.main()
